/*
 *    Copyright (C) 2002  Andrea Marchesini - Bakunin - bakunin@autistici.org
 *                        http://autistici.org/bakunin/
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#ifndef _MALLOC_H 
#define _MALLOC_H

#define TIME_MAX 10

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdarg.h>
#include <unistd.h>
#include <time.h>

#define GET(x,k) (x & k)

#define getrandom(min, max) ((rand() % (int)(((max)+1) - (min))) + (min))

#define LOCK getpid()
#define NOLOCK 0

typedef struct
{
  void *p;
  size_t size;
  unsigned char key;
} _malloc_;

typedef struct _malloc_2
{
  void *p;
  size_t size;
  _malloc_ *struct1;
  _malloc_ *struct2;
  unsigned char key;
  pid_t pid_lock;
  struct _malloc_2 *next;
} _malloc_list;

typedef struct
{
  size_t size;			/* Dimensione allocata */
  unsigned int zones;		/* Numero di segmenti allocati */
  unsigned int fulls;		/* Riempiti in toto */
  unsigned int fragments;	/* zone-full */
  unsigned int unuseds;		/* Zone inutilizzate della mallocazione */
} _malloc_state;

_malloc_list *_malloc_list_start;


/*****************
 * Funzioni Base *
 *****************/

/*
 * void _malloc_init(unsigned int) - inizializza la lista e in seme random 
 *                                   passato come argomento. 
 */
void _malloc_init (unsigned int);

/*
 * void _free(_malloc_ *) - libero lo spazio.
 */
void _free (_malloc_ *);

/*
 * int _malloc_change_key(_malloc_ *, const unsigned char) - Cambia la chiave
 *                                                           lettura di un di
 *                                                           _malloc_ e del suo
 *                                                           complementare.
 */
int _malloc_change_key (_malloc_ *, const unsigned char);


/***************
 * Allocazione *
 ***************/

/*
 * _malloc_ *_malloc(size_t) - _malloca la dimensione che si mette 
 *                             ovviamente * 2.
 */
_malloc_ *_malloc (size_t);

/*
 * _malloc_ *_realloc(_malloc_ *, size_t) - man realloc.
 */
_malloc_ * _realloc(_malloc_ *old, size_t size);

/*
 * _malloc_ *_calloc(size_t, size_t size) - man calloc.
 */
#define _calloc(a,b) _malloc((a)*(b))


/*******************
 * Accesso ai dati *
 *******************/

/*
 * void *_(_malloc_ *) - ritorna il contenuto in chiaro. Ovvio che quello
 *                       che ritorna deve subire subito una free.
 */
void *_return(_malloc_ *);

/*
 * int _fprintf(FILE *, const char *, ...) - uguale alla fprintf, supporta
 *                                           anche %_ che e' una struttura 
 *                                           _malloc_ stampandone il valore
 *                                           in chiaro.
 */
int _fprintf (FILE *, const char *, ...);

/*
 * int _printf(const char *, ...) - non e' nient'altro che una define di 
 *                                  questo tipo:
 *                               #define _printf(x...) _fprintf(strout,x)
 */
#define _printf(x...) _fprintf(stdout, x)


/*****************
 * Mem* e Simili *
 *****************/

/*
 * int _memcmp(const void *, const void *, size_t) - man memcmp. Lavora
 *                                                   in modo trasparente con
 *                                                   i _malloc_.
 */
int _memcmp (const void *, const void *, size_t);

/*
 * void *_memccpy(const void *, const void *, int, size_t) - man memccpy
 */
void *_memccpy (const void *, const void *, int, size_t);

/*
 * void *_mempcpy(const void *, const void *, size_t) - man mempcpy.
 */
void *_mempcpy (const void *, const void *, size_t);

/*
 * void *_memcpy(const void *, const void *, size_t) - non e' altro che:
 *                            #define _memcpy(a,b,d) _memccpy(a,b,0,d)
 */
#define _memcpy(a,b,c) _memccpy(a,b,0,c)

/*
 * void *_memset(const void *, int, size_t) - man memset. 
 */
void *_memset (void *, int, size_t);

/*
 * void *_memmove(const void *, const void *, size_t) - non e' altro che:
 *                              #define _memmove(a,b,d) _memccpy(a,b,0,d)
 */
#define _memmove(a,b,c) _memccpy(a,b,0,c)

/*
 * void *_memchr(const void *, int, size_t) - man memchr. 
 */
void *_memchr (const void *, int, size_t);

/*
 * void *_memrchr(const void *, int, size_t) - man memrchr.
 */
void *_memrchr (const void *, int, size_t);

/*
 * void *_rawmemchr(const void *, int) - man rawmemchr. 
 */
void *_rawmemchr (const void *, int);

/*
 * void *_memmem(const void *, size_t, const void *, size_t) - man memmem.
 */
void *_memmem (const void *, size_t, const void *, size_t);
#define _bcopy(a,b,c) _memcpy(b,a,n)
#define _bzero(a,b)   _memset(a,0,b)
#define _bcmp(a,b,c)  _memcmp(a,b,c)


/********************
 * Funzioni Interne *
 ********************/

/*
 * _malloc_list *_malloc_search(_malloc_ *) - funzione interna della struttura 
 *                                            associata ad uno spazio.
 */
_malloc_list *_malloc_search (_malloc_ *);

/*
 * _malloc_list *_malloc_search_free(size_t) - funzione interna di ricerca 
 *                                             spazio libero per il malloc.
 */
_malloc_list *_malloc_search_free (size_t);

/*
 * void _convert(char *, int, int) - funzione interna adibita alla 
 *                                   conversione di numeri utile alla 
 *                                   _fprintf.
 */
void _convert (char *, int, int);

/*
 * void *rand_malloc(size_t) -  una rivisitazione della rand_malloc di 
 *                             vecna.
 */
void *rand_malloc (size_t);


/*********************
 * Funzioni di Debug *
 *********************/

#ifdef DEBUG
int _malloc_debug;
int _malloc_debug_fd;

/*
 * void _malloc_set_debug(unsigned int) - Se si passa 0 si disattiva il debug,
 *                                        altrimenti si attiva con quel fattore 
 *                                        di verbosita'. Esistono 3 livelli di 
 *                                        verbosit:
 *
 *                                        0 -> Disattivo
 *                                        1 -> Funzioni di allocazione
 *                                        2 -> Ogni passaggio
 */
void _malloc_set_debug (unsigned int);

/*
 * void _malloc_set_fd(int) - Se si passa un file descriptor redireziona il 
 *                            debugging li'.
 */
void _malloc_set_fd (int);

/* 
 * void _malloc_log(const char *,...) - Scrive il log di debug
 */
void _malloc_log (const char *, ...);

/*
 * int _malloc_get_state(_malloc_state *) - Riempe una struttura passata come
 *                                          argomento con tutti i dati dello
 *                                          stato di _mallocazione. Guardare
 *                                          i campi della struttura. Torna 0 se
 *                                          non ci sono problemi, altrimenti 
 *                                          torna 1.
 */
#endif

int _malloc_get_state (_malloc_state *);

/*
 * int _malloc_get_entropy_key(const unsigned char) - Ritorna un numero da 0
 *                                                    a 15 specchio di un
 *                                                    calcolo di entropia sulla
 *                                                    variabile.
 */
int _malloc_get_entropy_key(const unsigned char);

#endif

/* EOF */
