/*
 *    Copyright (C) 2002  Andrea Marchesini - Bakunin - bakunin@autistici.org
 *                        http://autistici.org/bakunin/
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include <sys/types.h>
#include <_malloc.h>

void _malloc_init(unsigned int seed) {

  srand(seed);
  _malloc_start=_malloc_stop=NULL;
}

_malloc_ *_malloc(size_t size) {
  _malloc_list *tmp_list;
  _malloc_ *tmp_malloc;

  int fl;
  unsigned char key;

  tmp_list=_malloc_search_free(size);
  if(tmp_list) {
    if(!(tmp_malloc=(_malloc_ *)rand_malloc(sizeof(_malloc_)))) return NULL;

    if(!tmp_list->flag) tmp_malloc->key=~tmp_list->key;
    else                tmp_malloc->key=tmp_list->key;

    tmp_malloc->p=tmp_list->p;
    tmp_list->flag=2;

    return tmp_malloc;
  } 

  if(!(tmp_list=(_malloc_list *)rand_malloc(sizeof(_malloc_list)))) return NULL;

  if(!(tmp_list->p=(void *)rand_malloc(size*2))) {
    free(tmp_list);
    return NULL;
  }

  if(!(tmp_malloc=(_malloc_ *)rand_malloc(sizeof(_malloc_)))) {
    free(tmp_list->p);
    free(tmp_list);
    return NULL;
  }

  key=getrandom(1,255);

  tmp_malloc->key=key;
  tmp_malloc->p=tmp_list->p;

  tmp_list->key=key;
  tmp_list->flag=0;
  tmp_list->size=size;
  tmp_list->next=NULL;
 
  if(_malloc_start==NULL) {
    _malloc_stop=NULL;

    _malloc_start=_malloc_stop=tmp_list;
    return tmp_malloc;
  } 

  _malloc_stop->next=tmp_list;
  _malloc_stop=tmp_list;
  return tmp_malloc;
}

void _free(_malloc_ *tmp) {
 
  _malloc_list *tmp_list, *tmp2;

  if(tmp==NULL)           return;
  if(_malloc_stop==NULL)  return;
  if(_malloc_start==NULL) return;

  tmp_list=_malloc_search(tmp->p);

  if(!tmp_list) return;

  if(tmp_list->flag!=2) {
    if(tmp_list->flag!=(tmp->key & 1)) return;

    if(tmp_list==_malloc_start) {

      _malloc_start=tmp_list->next;

      free(tmp_list->p);
      free(tmp_list);
      free(tmp);

      return;
    }

    tmp2=_malloc_start;

    while(tmp2->next) {
      if(tmp2->next->p==tmp_list->p) {
        if(tmp2->next==_malloc_stop) {
          tmp2->next=NULL;
          _malloc_stop=tmp2;
          free(tmp_list->p);
          free(tmp_list);

          return;
        }
        tmp2->next=tmp_list->next;
        free(tmp_list->p);
        free(tmp_list);
        return;
      }
      tmp2=tmp2->next;
    }

    _malloc_stop=tmp2;
    free(tmp_list);

    return;
  }

  if((tmp->key & 1)) tmp_list->flag=1;
  else               tmp_list->flag=0;
  free(tmp);
}

_malloc_list *_malloc_search(void *p) {
  _malloc_list *tmp;

  if(p==NULL) return NULL;
  if(_malloc_start==NULL) return NULL;

  tmp=_malloc_start;

  while(tmp) {
    if(tmp->p==p) return tmp;
    tmp=tmp->next;
  }
  return NULL;
}

_malloc_list *_malloc_search_struct(_malloc_ *p) {
  _malloc_list *tmp;

  if(!p) return NULL;
  if(!_malloc_start) return NULL;

  tmp=_malloc_start;

  while(tmp) {
    if(tmp->p==p->p) return tmp;
    tmp=tmp->next;
  }
  return NULL;
}

_malloc_list *_malloc_search_free(size_t size) {
  _malloc_list *tmp;

  if(!_malloc_start) return NULL;

  tmp=_malloc_start;

  while(tmp) {
    if(tmp->flag!=2 && tmp->size>=size) return tmp;
    tmp=tmp->next;
  }
  return NULL;
}

void *_return(_malloc_ *list) {
  _malloc_list *tmp;
  char *str, *ch;
  int size;

  tmp=_malloc_search(list->p);
  if(!tmp) return NULL;

  if(!(str=(char *)rand_malloc((tmp->size+1)*sizeof(char)))) return NULL;

  ch=(char *)list->p;

  for(size=0;size<tmp->size;size++) {
    *(str+size)=GET(*(ch+(size*2)),list->key) | GET(*(ch+((size*2)+1)),~list->key);
  }  

  str[size]=0;
  return (void *)str;
}

void *rand_malloc(size_t size) {
  void *foo, *ret;
  static int foospace=32;

  foospace *= getrandom(2,256);
  foospace %= 256;

  foo = malloc(foospace);
  if(!foo) return NULL;

  ret = malloc(size);
  if(!ret) return NULL;

  memset(ret, 0x00, size);

  free(foo);

  return ret;
}
