/*
 *    Copyright (C) 2002  Andrea Marchesini - Bakunin - bakunin@autistici.org
 *                        http://autistici.org/bakunin/
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include <sys/types.h>
#include <_malloc.h>

void *
_memccpy (const void *s1, const void *s2, int c, size_t n)
{

  _malloc_list *list1=NULL, *list2=NULL;
  _malloc_ *tmp1, *tmp2;
  unsigned char *dest;
  unsigned char *src;
  unsigned char flag = 0;

/*
  A e B sono i 2 void * e questi sono i valori possibili di flag per capire se
  sono o no delle zone _malloccate.

  B A
  0 0 -> 0x0
  0 1 -> 0x1
  1 0 -> 0x2
  1 1 -> 0x3
*/

  if ((list1=_malloc_search ((_malloc_ *) s1)))
    {

      if(check_locking(list1)) return NULL;

      list1->pid_lock=LOCK;

#ifdef DEBUG
      if (_malloc_debug > 1)
	_malloc_log ("_memccpy: Primo argomento e' _malloc_.\n");
#endif

      tmp1 = (_malloc_ *) s1;
      dest = (char *) tmp1->p;

      flag |= 0x1;

    }
  else
    {

#ifdef DEBUG
      if (_malloc_debug > 1)
	_malloc_log ("_memccpy: Primo argomento e' in chiaro.\n");
#endif

      dest = (char *) s1;
    }

  if ((list2=_malloc_search ((_malloc_ *) s2)))
    {

      if(check_locking(list2)) return NULL;

      list2->pid_lock=NOLOCK;

#ifdef DEBUG
      if (_malloc_debug > 1)
	_malloc_log ("_memccpy: Secondo argomento e' _malloc_.\n");
#endif

      tmp2 = (_malloc_ *) s2;
      src = (char *) tmp2->p;

      flag |= 0x2;

    }
  else
    {

#ifdef DEBUG
      if (_malloc_debug > 1)
	_malloc_log ("_memccpy: Secondo argomento e' in chiaro.\n");
#endif

      src = (char *) s2;
    }

  switch (flag)
    {
    case 0x0:

      while (n-- && src[n] != c)
	dest[n] = src[n];

      break;
    case 0x1:

      for (; n; n--)
	{

	  if (src[n - 1] == c)
	    break;

	  dest[(n * 2) - 1] &= tmp1->key;
	  dest[(n - 1) * 2] &= ~tmp1->key;

	  dest[(n * 2) - 1] |= GET (src[n - 1], ~tmp1->key);
	  dest[(n - 1) * 2] |= GET (src[n - 1], tmp1->key);
	}

      break;
    case 0x2:

      while (n--)
	{

	  if ((GET (src[n * 2], tmp2->key) +
	       GET (src[(n * 2) + 1], ~tmp2->key)) == c)
	    break;

	  dest[n] =
	    GET (src[n * 2], tmp2->key) + GET (src[(n * 2) + 1], ~tmp2->key);
	}

      break;
    case 0x3:

      for (; n; n--)
	{

	  if ((GET (src[(n * 2) - 1], ~tmp2->key) |
	       GET (src[(n - 1) * 2], tmp2->key)) == c)
	    break;

	  dest[(n * 2) - 1] &= tmp1->key;
	  dest[(n - 1) * 2] &= ~tmp1->key;

	  flag =
	    GET (src[(n * 2) - 1],
		 ~tmp2->key) | GET (src[(n - 1) * 2], tmp2->key);

	  dest[(n * 2) - 1] |= GET (flag, ~tmp1->key);
	  dest[(n - 1) * 2] |= GET (flag, tmp1->key);
	}

      break;
    }

  if(list1) list1->pid_lock=NOLOCK;
  if(list2) list2->pid_lock=NOLOCK;

  return (void *) dest;
}
