/*
 *    Copyright (C) 2002  Andrea Marchesini - Bakunin - bakunin@autistici.org
 *                        http://autistici.org/bakunin/
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include <sys/types.h>
#include <_malloc.h>

void *
_memmem (const void *s1, size_t s1_len, const void *s2, size_t s2_len)
{
  _malloc_list *list=NULL;
  char *st1, *st2;
  char *str1 = NULL, *str2 = NULL;

  if ((list=_malloc_search ((_malloc_ *) s1)))
    {

      if(check_locking(list)) return NULL;

#ifdef DEBUG
      if (_malloc_debug > 1)
	_malloc_log ("_memmem: Primo argomento e' _malloc_.\n");
#endif

      str1 = st1 = _return((_malloc_ *) s1);

    }
  else
    {

#ifdef DEBUG
      if (_malloc_debug > 1)
	_malloc_log ("_memmem: Primo argomento e' in chiaro.\n");
#endif

      st1 = (char *) s1;
    }

  if ((list=_malloc_search ((_malloc_ *) s2)))
    {

      if(check_locking(list)) return NULL;

#ifdef DEBUG
      if (_malloc_debug > 1)
	_malloc_log ("_memmem: Secondo argomento e' _malloc_.\n");
#endif

      str2 = st2 = _return((_malloc_ *) s2);
    }
  else
    {

#ifdef DEBUG
      if (_malloc_debug > 1)
	_malloc_log ("_memmem: Secondo argomento e' in chiaro.\n");
#endif

      st2 = (char *) s2;

    }

  while (s1_len && st1)
    {

      s1_len--;

      if (*st1 == *st2)
	{
	  char *tmp1, *tmp2;
	  int i, k;

	  tmp1 = st1;
	  tmp2 = st2;

	  i = s1_len;
	  k = s2_len;

	  while (i-- && k--)
	    {

	      if (*tmp1 != *tmp2)
		break;

	      tmp1++;
	      tmp2++;
	    }

	  if (!k || !i)
	    {
	      if (str2)
		free (str2);

	      if (str1)
		{

		  _malloc_ *tmp;
		  tmp = (_malloc_ *) s1;

		  i = st1 - str1;
		  free (str1);

		  return (void *) tmp->p + i;
		}

	      return st1;
	    }
	}

      st1++;
    }

  if (str1)
    free (str1);
  if (str2)
    free (str2);

  return NULL;
}
