/*
 *    Copyright (C) 2002  Andrea Marchesini - Bakunin - bakunin@autistici.org
 *                        http://autistici.org/bakunin/
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include <sys/types.h>
#include <_malloc.h>

_malloc_ *
_malloc (size_t size)
{
  _malloc_list *tmp_list;
  _malloc_ *tmp_malloc;

  int fl;
  unsigned char key;

  /* Cerco uno spazio libero in parte */
  tmp_list = _malloc_search_free (size);

  if (tmp_list)
    {

      /* Checco un eventuale lock */
      if(check_locking(tmp_list)) return;

      tmp_list->pid_lock=LOCK;

      /* Alloco una struttura _malloc_ da ritornare */
      if (!(tmp_malloc = (_malloc_ *) rand_malloc (sizeof (_malloc_))))
	return NULL;

      tmp_malloc->p = tmp_list->p;
      tmp_malloc->size = size;

      /* Setto come occupato questo spazio */
      if (tmp_list->struct1)
	{
	  tmp_list->struct2 = tmp_malloc;
	  tmp_malloc->key = ~tmp_list->key;
	}
      else
	{
	  tmp_malloc->key = tmp_list->key;
	  tmp_list->struct1 = tmp_malloc;
	}

#ifdef DEBUG
      if (_malloc_debug)
	_malloc_log
	  ("_malloc: Nuova _malloc_ su spazio gia' allocato: %p key: %X size: %d size zone: %d.\n",
	   tmp_malloc, tmp_malloc->key, size, tmp_list->size);
#endif

      tmp_list->pid_lock=NOLOCK;

      return tmp_malloc;
    }

  /* Alloco una serie di cose */
  if (!(tmp_list = (_malloc_list *) rand_malloc (sizeof (_malloc_list))))
    return NULL;

  if (!(tmp_list->p = (void *) rand_malloc (size * 2)))
    {
      free (tmp_list);

#ifdef DEBUG
      if (_malloc_debug > 1)
	_malloc_log ("_malloc: Errore di allocazione.\n");
#endif

      return NULL;
    }

  /* Nel mentre locko */
  tmp_list->pid_lock=LOCK;

  if (!(tmp_malloc = (_malloc_ *) rand_malloc (sizeof (_malloc_))))
    {
      free (tmp_list->p);
      free (tmp_list);

#ifdef DEBUG
      if (_malloc_debug > 1)
	_malloc_log ("_malloc: Errore di allocazione.\n");
#endif

      return NULL;
    }

  key = getrandom (0x2, 0xFF);

  tmp_malloc->key = key;
  tmp_malloc->p = tmp_list->p;
  tmp_malloc->size = size;
  tmp_list->key = key;

  tmp_list->struct1 = tmp_malloc;
  tmp_list->struct2 = NULL;

  tmp_list->size = size;
  tmp_list->next = NULL;

#ifdef DEBUG
  if (_malloc_debug)
    _malloc_log
      ("_malloc: Nuova _malloc_ su nuovo spazio: %p key: %X size: %d.\n",
       tmp_malloc, tmp_malloc->key, size);
#endif

  /* Aggiungo l'elemento alla lista */
  if (_malloc_list_start == NULL)
    {
      _malloc_list_start = tmp_list;

      tmp_list->pid_lock=NOLOCK;

      return tmp_malloc;
    }

  tmp_list->next = _malloc_list_start;
  _malloc_list_start = tmp_list;

  tmp_list->pid_lock=NOLOCK;

  return tmp_malloc;
}

_malloc_ *
_realloc (_malloc_ * old, size_t size)
{
  _malloc_ *new;

#ifdef DEBUG
  if (_malloc_debug)
    _malloc_log ("_realloc: size of %p: %d\n", old, old->size);
#endif

  /* Alloco uno spazio nuovo */
  if (!(new = _malloc (size)))
    return NULL;

#ifdef DEBUG
  if (_malloc_debug)
    _malloc_log ("_realloc: size of %p: %d\n", new, new->size);
#endif

  /* Copio il contenuto e libero */
  _memcpy (new, old, old->size);
  _free (old);

  return new;
}
