/*
 *    Copyright (C) 2002  Andrea Marchesini - Bakunin - bakunin@autistici.org
 *                        http://autistici.org/bakunin/
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include <sys/types.h>
#include <_malloc.h>

void
_malloc_init (const unsigned int seed)
{

#ifdef DEBUG
  if (_malloc_debug)
    _malloc_log ("_malloc_init: inizializzazione sistema.\n");

  _malloc_debug = 0;
  _malloc_debug_fd = STDERR_FILENO;
#endif

  /* Inizializzo il seme e la lista */
  srand (seed);

  _malloc_list_start = NULL;

}

void
_free (_malloc_ * tmp)
{

  _malloc_list *tmp_list, *tmp2;

#ifdef DEBUG
  if (_malloc_debug)
    _malloc_log ("_free: Libero _malloc_: %p.\n", tmp);
#endif

  if (tmp == NULL)
    return;
  if (_malloc_list_start == NULL)
    return;

  /* Cerco l'elemento da liberare */
  tmp_list = _malloc_search (tmp);

  if (!tmp_list)
    return;

  /* Checco un eventuale locking, e poi se non..., lo setto io */
  if(check_locking(tmp_list)) 
    return;

  tmp_list->pid_lock=LOCK;

  _memset (tmp, 0, tmp_list->size);

  /* Elimino elemento nella lista */
  if (!tmp_list->struct1 || !tmp_list->struct2)
    {

      /* Se e' all'inizio */
      if (tmp_list == _malloc_list_start)
	{

	  _malloc_list_start = tmp_list->next;

	  free (tmp_list->p);
	  free (tmp_list);
	  free (tmp);

	  return;
	}

      /* Se e' da altre parti */
      tmp2 = _malloc_list_start;

      while (tmp2->next)
	{

          if(check_locking(tmp2)) return;

	  if (tmp2->next == tmp_list)
	    {

              tmp2->pid_lock=LOCK;

	      tmp2->next = tmp_list->next;

              tmp2->pid_lock=NOLOCK;

	      free (tmp_list->p);
	      free (tmp_list);
	      return;
	    }

	  tmp2 = tmp2->next;
	}

      return;
    }

  /* In caso non siano occupati entrambi gli elementi, elimino uno dei 2 */

  tmp_list->pid_lock=LOCK;

  if (tmp_list->struct1 == tmp)
    tmp_list->struct1 = NULL;
  else
    tmp_list->struct2 = NULL;

  tmp_list->pid_lock=NOLOCK;

  free (tmp);
}

/* Dato un indirizzo, ritorno la struttura */
_malloc_list *
_malloc_search (_malloc_ *p)
{
  _malloc_list *tmp;

#ifdef DEBUG
  if (_malloc_debug > 2)
    _malloc_log
      ("_malloc_search: Cerco la _malloc_list legata a _malloc_: %p.\n",
       p);
#endif

  if (!p)
    return NULL;
  if (!_malloc_list_start)
    return NULL;

  tmp = _malloc_list_start;

  while (tmp)
    {
      if (tmp->struct1 == p || tmp->struct2 == p)
	return tmp;
      tmp = tmp->next;
    }
  return NULL;
}

/* Cerco uno spazio libero */
_malloc_list *
_malloc_search_free (size_t size)
{
  _malloc_list *tmp;

#ifdef DEBUG
  if (_malloc_debug > 2)
    _malloc_log
      ("_malloc_search_free: Cerco la _malloc_list libera e di dimensione: %d.\n",
       size);
#endif

  if (!_malloc_list_start)
    return NULL;

  tmp = _malloc_list_start;

  while (tmp)
    {
      if ((!tmp->struct1 || !tmp->struct2) && tmp->size >= size)
	return tmp;
      tmp = tmp->next;
    }
  return NULL;
}

void *
rand_malloc (size_t size)
{
  void *foo, *ret;
  static int foospace = 0x20;

  foospace *= getrandom (0x2, 0xFF);
  foospace %= 0xFF;

#ifdef DEBUG
  if (_malloc_debug > 1)
    _malloc_log
      ("_rand_malloc: Creo spazio distanziatore di dimensione: %d.\n",
       foospace);
#endif

  foo = malloc (foospace);
  if (!foo)
    return NULL;

#ifdef DEBUG
  if (_malloc_debug > 1)
    _malloc_log
      ("_rand_malloc: Creo spazio richiesto di dimensione: %d.\n", size);
#endif

  ret = malloc (size);
  if (!ret)
    return NULL;

  memset (ret, 0x00, size);

  free (foo);

  return ret;
}

int check_locking(_malloc_list *list) {
  time_t t;

  if(!list) return 0;

#ifdef DEBUG
  if(_malloc_debug>1) 
    _malloc_log ("check_locking: PID of %p: %d.\n", list, list->pid_lock);
#endif

  t=time(NULL);

  do {

    if(!list->pid_lock || list->pid_lock==LOCK) 
      return 0;

  } while(time(NULL)-t<TIME_MAX);

#ifdef DEBUG
  if(_malloc_debug>1) 
    _malloc_log ("check_locking: TimeOut.\n");
#endif

  return 1;
}
