/*
 *    Copyright (C) 2002  Andrea Marchesini - Bakunin - bakunin@autistici.org
 *                        http://autistici.org/bakunin/
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include <_malloc.h>

int
_malloc_change_key (_malloc_ * str, const unsigned char key)
{
  _malloc_list *tmp_list, *tmp_new, *tmp;
  char *p, *dest;
  int n;

#ifdef DEBUG
  if (_malloc_debug)
    _malloc_log ("_malloc_change_key: Setto la _malloc_ %p con key %X.\n",
		 str, key);
#endif

  if (!str)
    return 1;

  tmp_list = _malloc_search (str);
  if (!tmp_list)
    return 1;

  if(check_locking(tmp_list)) return;

  tmp_list->pid_lock=LOCK;

  p = _return(str);

  str->key = key;

  if (!(str->p = (void *) rand_malloc (str->size * 2)))
    {
      return 1;
    }

#ifdef DEBUG
  if (_malloc_debug > 2)
    _malloc_log ("_malloc_change_key: Generato nuovo spazio %p.\n", str->p);
#endif

  if (!(tmp_new = (void *) rand_malloc (sizeof (_malloc_list))))
    {
      free (str->p);
      return 1;
    }

#ifdef DEBUG
  if (_malloc_debug > 2)
    _malloc_log ("_malloc_change_key: Creata nuova _malloc_list %p.\n", tmp_new);
#endif

  tmp_new->pid_lock=LOCK;
  tmp_new->next = NULL;
  tmp_new->p = str->p;
  tmp_new->key = key;
  tmp_new->size = str->size;

  tmp_new->struct1 = str;
  tmp_new->struct2 = NULL;

#ifdef DEBUG
  if (_malloc_debug > 2)
    _malloc_log
      ("_malloc_change_key: Setto valori dentro al nuovo spazio.\n");
#endif

  dest = (char *) str->p;
  for (n = tmp_new->size; n; n--)
    {

      dest[(n * 2) - 1] &= key;
      dest[(n - 1) * 2] &= ~key;

      dest[(n * 2) - 1] |= GET (p[n - 1], ~key);
      dest[(n - 1) * 2] |= GET (p[n - 1], key);
    }
  free (p);

#ifdef DEBUG
  if (_malloc_debug > 2)
    _malloc_log ("_malloc_change_key: Ricreazione lista.\n");
#endif

  tmp_new->next = _malloc_list_start;
  _malloc_list_start = tmp_new;

  if (!tmp_list->struct1 || !tmp_list->struct2)
    {

      tmp = _malloc_list_start;

      while (tmp->next)
	{
          if(check_locking(tmp)) {

            _malloc_list_start=tmp_new->next;
            free(tmp_new->p);
            free(tmp_new);

            return 1;
          }

	  if (tmp->next == tmp_list)
	    {

              tmp->pid_lock=LOCK;

	      tmp->next = tmp_list->next;

              tmp->pid_lock=NOLOCK;

	      free (tmp_list->p);
	      free (tmp_list);

              tmp_new->pid_lock=NOLOCK;

	      return 0;
	    }
	  tmp = tmp->next;
	}

      tmp_new->pid_lock=NOLOCK;

      return 0;
    }

  if (tmp_list->struct1 == str)
    tmp_list->struct1 = NULL;
  else
    tmp_list->struct2 = NULL;

  tmp_list->pid_lock=NOLOCK;
 
  tmp_new->pid_lock=NOLOCK;

  return 0;
}

int
_malloc_get_entropy_key (const unsigned char key)
{
  unsigned char tmp1, tmp2;

  tmp1 = ((key & 0x80) >> 4) |
    ((key & 0x20) >> 3) | ((key & 0x08) >> 2) | ((key & 0x02) >> 1);

  tmp2 = ((key & 0x40) >> 3) |
    ((key & 0x10) >> 2) | ((key & 0x04) >> 1) | (key & 0x1);

  tmp1 = abs (tmp1 - tmp2);

#ifdef DEBUG
  if (_malloc_debug)
    _malloc_log ("_malloc_get_entropy_key: Entropia della key %X: %d.\n", key,
		 tmp1);
#endif

  return tmp1;
}

unsigned char
_malloc_new_key (void)
{

  return (unsigned char) getrandom (0x2, 0xFF);
}
