/*
 *    Copyright (C) 2002  Andrea Marchesini - Bakunin - bakunin@autistici.org
 *                        http://autistici.org/bakunin/
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include <sys/types.h>
#include <_malloc.h>

void *
_return(_malloc_ * list)
{
  char *str, *ch;
  int size;

#ifdef DEBUG
  if (_malloc_debug)
    _malloc_log ("_: Ritorno il contenuto in chiaro di _malloc_: %p.\n",
		 list);
#endif

  if (!(str = (char *) rand_malloc ((list->size + 1) * sizeof (char))))
    return NULL;

  ch = (char *) list->p;

  for (size = 0; size < list->size; size++)
    {
      *(str + size) =
	GET (*(ch + (size * 2)),
	     list->key) | GET (*(ch + ((size * 2) + 1)), ~list->key);
    }

  str[size] = 0;
  return (void *) str;
}

int
_fprintf (FILE * stream, const char *format, ...)
{

  char **arg = (char **) &format;
  int c, len = 0;
  char buf[20];

  while ((c = *format++) != 0)
    {

      if (c != '%')
	{
	  len++;
	  putc (c, stream);
	}
      else
	{

	  char *p;
	  c = *format++;
	  switch (c)
	    {

	    case '_':
	      {
		char *a;

		a = p = (char *) _return((_malloc_ *) * (++arg));

		while (*p)
		  {
		    len++;
		    putc (*p++, stream);
		  }

		free (a);

		break;
	      }

	    case 'p':
	      len += fprintf (stream, "%p", *(++arg));
	      break;

	    case 'c':
	      len += fprintf (stream, "%c", *(++arg));
	      break;

	    case 'd':
	    case 'u':
	    case 'x':
	    case 'X':
	      _convert (buf, c, *((int *) ++arg));

	      if (c == 'x' || c == 'X')
		{
		  putc ('0', stream);
		  putc ('x', stream);
		}

	      p = buf;
	      while (*p)
		{
		  len++;
		  putc (*p++, stream);
		}
	      break;

	    case 's':
	      p = *(++arg);
	      if (!p)
		p = "(null)";
	      while (*p)
		{
		  len++;
		  putc (*p++, stream);
		}
	      break;

	    default:
	      len++;
	      putc (c, stream);
	      break;
	    }
	}
    }

  return len;
}

void
_convert (char *buf, int base, int d)
{
  char l = 'a';
  char *p = buf;
  char *p1, *p2;
  unsigned long ud = d;
  int divisor = 10;

  if (base == 'd' && d < 0)
    {
      *p++ = '-';
      buf++;
      ud = -d;
    }
  else if (base == 'x')
    divisor = 16;
  else if (base == 'X')
    {
      divisor = 16;
      l = 'A';
    }

  do
    {
      int remainder = ud % divisor;
      *p++ = (remainder < 10) ? remainder + '0' : remainder + l - 10;
    }

  while (ud /= divisor);

  *p = 0;

  p1 = buf;
  p2 = p - 1;
  while (p1 < p2)
    {
      char tmp = *p1;
      *p1 = *p2;
      *p2 = tmp;
      p1++;
      p2--;
    }
}
