/*
 * Copyright (C) 2003  
 * paranoy project - paranox.sf.net - paranoy@autistici.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include "include.h"

 /*
  * generate a random sequence
  */
char *
gen_random(int len)
{
  struct timeval tv;
  char *buff = (char *) malloc(sizeof(char) * (len + 1));

  gettimeofday(&tv, 0);
  RAND_load_file("/dev/urandom", len);
  RAND_seed((const char *) &tv, sizeof(tv));
  RAND_bytes(buff, len);
  RAND_cleanup();
  return buff;
}


// generate the md5 hash of the argouments 
char *
MD5String(char *str)
{
  ushort i;
  MD5_CTX context;
  unsigned char digest[16];
  uint len = strlen(str);

  MD5_Init(&context);
  MD5_Update(&context, str, len);
  MD5_Final(digest, &context);

  for (i = 0; i < 16; i++)
    sprintf(str + 2 * i, "%02x", digest[i]);
  str[32] = 0;
  return (str);
}


/*
 * add global var "EVP_CIPHER *def_cp", that contains the user's preferred
 * cipher algorithm. (blowfish, idea, des, ...) 
 */

EVP_CIPHER *def_cp = NULL;

/*
 * add a #define DEF_ALGO, defined at compile time, that contains the default 
 * cipher algorithm 
 */

#define DEF_ALGO EVP_bf_cbc()

/*
 * [en|de]crypt using cipher "algo". enc: 1 encrypt, 0 decrypt. Don't use to
 * write a file: use cfwrite() instead. 
 */
int
cryptz(char *algo, unsigned char *in, int len, unsigned char *out,
       unsigned char *key, int enc)
{
  EVP_CIPHER *cp;
  EVP_CIPHER_CTX ctx;
  int outlen, tmplen;
  char iv[9] = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };

  EVP_CIPHER_CTX_init(&ctx);


  if ((cp = (EVP_CIPHER *) EVP_get_cipherbyname(algo)) == NULL)
  {
    logg("Unknown cipher: %s; using default.\n", algo);
    cp = (EVP_CIPHER *) DEF_ALGO;
  }
  EVP_CipherInit(&ctx, cp, NULL, NULL, enc);
  EVP_CIPHER_CTX_set_key_length(&ctx, strlen(key));
  EVP_CipherInit(&ctx, NULL, key, iv, enc);

  if (!EVP_CipherUpdate(&ctx, out, &outlen, in, len))
    return (0);

  if (!EVP_CipherFinal(&ctx, out + outlen, &tmplen))
    return (0);

  outlen += tmplen;
  EVP_CIPHER_CTX_cleanup(&ctx);
  return (outlen);
}

/*
 * You need to call this *before* using cfread() and cfwrite(). 
 */
int
crypto_init(EVP_CIPHER_CTX * ctx, int enc)
{
  EVP_CIPHER *cp = NULL;
  char iv[9] = { 1, 2, 3, 4, 5, 6, 7, 8, 0 };
  char key[18] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

  // char *key = _(_random);
  EVP_CIPHER_CTX_init(ctx);

  /*
   * if there's not user's preference, we use default 
   */
  if (def_cp == NULL)
    cp = (EVP_CIPHER *) DEF_ALGO;


  logg("crypt - dentro crypto_init ctx = %p\n", ctx);
  /*
   * it's strange but we have to do so. See openssl doc. 
   */
  logg("crypt - prima di cipher_init cp = %p , enc = %d\n", cp, enc);
  EVP_CipherInit_ex(ctx, cp, NULL, NULL, NULL, enc);
  logg("crypt - prima di lanciare set_key_lenght\n");
  EVP_CIPHER_CTX_set_key_length(ctx, KEY_LEN);
  // logg("crypt - al cyper_init ctx = %p , key = %p - '%s' , iv = %p\n",
  // ctx,
  // key, key, iv);
  EVP_CipherInit_ex(ctx, NULL, NULL, key, iv, enc);
  logg("dopo cipher_init\n");
  return 1;
}

/*
 * encrypts and writes
 inbuf to file f. Returns the number of bytes
 * successfully encrypted. 
 */
int
cfwrite(EVP_CIPHER_CTX * ctx, char *inbuf, int len, int last, FILE * f)
{
  char buff[BUFFER_REAL];
  int outlen;

  if (!EVP_EncryptUpdate(ctx, buff, &outlen, inbuf, len))
  {
    printf("Error while writing...\n");
    return 0;
  }
  if (last)
  {
    int templen;

    logg("crypt - prima di encryptfinal\n");
    if (!EVP_EncryptFinal_ex(ctx, buff + outlen, &templen))
    {
      printf("error while encrypfinal\n");
      return 0;
    }
    outlen += templen;
  }

  fwrite(buff, 1, outlen, f);
  if (last)
  {
    logg("cfwrite - LIBERO il context\n");
    EVP_CIPHER_CTX_cleanup(ctx);
    fclose(f);
  }
  return outlen;
}

/*
 * read len bytes from file f. Returns the number of
 * bytes successfully decrypted 
 */
int
cfread(EVP_CIPHER_CTX * ctx, char *outbuf, FILE * f)
{
  char buff[BUFFER_REAL];
  int inlen, outlen;

  inlen = fread(buff, 1, BUFFER_LEN, f);
  if (inlen <= 0)
  {
    if (!EVP_DecryptFinal(ctx, outbuf, &outlen))
    {
      return 0;
    }
    outbuf[outlen] = 0;
    EVP_CIPHER_CTX_cleanup(ctx);
    fclose(f);
    return 0;
  }

  if (!EVP_DecryptUpdate(ctx, outbuf, &outlen, buff, inlen))
    return 0;

  outbuf[outlen] = 0;
  return outlen;

}

int
readcline(EVP_CIPHER_CTX * ctx, FILE * file, char *inbuf, int maxlen,
          ushort * len)
{
  int n, rc;
  char c, *ptr;

  ptr = inbuf;
  *ptr = 0;
  for (n = 1; n < maxlen; n++)
  {
    rc = readcch(ctx, file, &c);
    if (rc == 1)
    {
      if (c == '\n')
        break;
      *ptr++ = c;
    }
    // end of file
    else
    {
      logg("il file e' finito, andate in pace\n");
      *ptr = 0;
      if (len)
        *len = n - 1;
      return 0;
    }
  }
  *ptr = 0;
  if (len)
    *len = n - 1;
  return 1;
}


int
readcch(EVP_CIPHER_CTX * ctx, FILE * file, char *ptr)
{
  static int read_cnt = 0;
  static char *read_ptr;
  static byte flag = 0;
  static char read_buf[BUFFER_REAL];

  if (read_cnt <= 0)
  {
    if (flag)
    {
      read_cnt = 0;
      flag = 0;
      return 0;
    }
    if ((read_cnt = cfread(ctx, read_buf, file)) <= 0)
      flag = 1;
    read_cnt = strlen(read_buf);
    read_ptr = read_buf;
  }
  read_cnt--;
  *ptr = *read_ptr++;
  return 1;
}
