/*
 * Copyright (C) 2003  
 * paranoy project - paranoy.sf.net - paranoy@autistici.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include "include.h"


// called when paranoy receive a SIGINT
void
cursig()
{
  logg("Ha premuto contro+c\n");
  end("Signal received....send bugs report to paranoy@autistici.org");
}


void
end(const char *msg)
{
  // save all mails tree
  write_index();

  if (gui == NCURSES)
  {
#ifdef USE_CURSES
    if (stdscr != NULL)
    {
      info(0, "Closing....");
      usleep(10000);
      clear();
      refresh();
      endwin();
    }
#endif
  } else if (gui == GTK)
  {
#ifdef USE_GTK

#endif
  }
  fprintf(stderr, "%s\n", msg);
  exit(0);
}

// thread that lunchs download function each mail_check_time minutes
void
timed()
{
  ushort i;

  while (1)
  {
    if (mail_check_time == 0)
      return;

    for (i = 0; i < mail_check_time; i++, sleep(60));
    if (!downloading)
    {
      logg("dentro downloading\n");
      downloading = 1;
      pthread_create(&download_thread, NULL, (void *) download, NULL);
    }
  }
}


/*
 * strip all occurances of c from the string 
 */
void
strip(char *string, char c)
{
  char *src, *dst;

  if (!string)
    return;

  for (src = dst = string; *src; src++)
    if (*src != c)
      *dst++ = *src;
  *dst = '\0';
}

// investigate what protocol a server support (it choose the better)
byte
autoprobe(char *server, char *user, char *password, ushort * port)
{
  struct connection *c;
  c = (struct connection *) smalloc(sizeof(struct connection));
  c->sbio = NULL;
  c->err = NO_ERROR;


  /*
   * while probing for a valid protocol, try first pop3 + ssl, then imap +
   * ssl, clear pop3 and finally clear imap.. if nobody of these are founded
   * return 0 
   */

  info(0, "Probing pop3 in ssl...\n");
  if (!para_connect(c, server, POP3S_PORT, SSL_CONNECTION))
  {
    para_close(c);
    free(c);
    *port = POP3S_PORT;
    info(2, "POP3 in SSL is supported by %s!!Good.\n", server);
    return POP3_SSL_PROTO;
  }

  info(0, "Probing imap in ssl...\n");
  if (!para_connect(c, server, IMAPS_PORT, SSL_CONNECTION))
  {
    para_close(c);
    free(c);
    *port = IMAPS_PORT;
    info(2, "IMAP in SSL is supported by %s!!Good.\n", server);
    return IMAP_SSL_PROTO;
  }
  c->sbio = NULL;
  info(0, "Probing pop3 in clear mode...\n");
  if (!para_connect(c, server, POP3_PORT, CLEAR_CONNECTION))
  {
    para_close(c);
    free(c);
    *port = POP3_PORT;
    info(2, "Only clear text pop3 protocol is supported(WARNING)!\n");
    return POP3_CLEAR_PROTO;
  }
  // info(0,"Probing imap in clear mode...");
  if (!para_connect(c, server, IMAP_PORT, CLEAR_CONNECTION))
  {
    para_close(c);
    free(c);
    *port = IMAP_PORT;
    info(2, "Only clear text imap protocol is supported  (WARNING)!\n");

    return IMAP_CLEAR_PROTO;
  }
  info(2, "No valid protocol found!\n");
  return 0;
}

// return true if mail argoument is a valid mail address
byte
is_mail(char *mail)
{
  char *a;

  if (!mail)
    return 0;

  if ((a = strstr(mail, "@")) == NULL)
    return 0;

  if (strstr(a + 1, "@"))
    return 0;

  if ((a = strstr(a, ".")) == NULL)
    return 0;

  return 1;
}

// add email mail to folder fold
void
add_mail(struct folders *fold, struct email *mail)
{
  // the folder is empty
  if (!fold->head)
  {
    fold->head = (struct list *) smalloc(sizeof(struct list));
    fold->head->prev = NULL;
    fold->head->next = NULL;
    fold->head->mail = mail;
  } else
  {
    struct list *search, *l;

    search = fold->head;
    while (search && mail->inreplyto)
    {
      if (!strcmp(search->mail->messageid, mail->inreplyto))
        break;
      search = search->next;
    }

    l = (struct list *) malloc(sizeof(struct list));
    l->mail = mail;
    if (!search && mail->inreplyto)
    {
      l->next = fold->head;
      fold->head->prev = l;
      l->prev = NULL;
      fold->head = l;
    } else
    {
      l->mail->level = search->mail->level + 1;
      if (search->next)
        search->next->prev = l;
      l->next = search->next;
      search->next = l;
      l->prev = search;
    }
  }

  fold->nall++;
  fold->nnew++;
}

// move a mail from a folder to another one
byte
move_mail(const char *from, const char *to, struct list *mail)
{
  struct folders *ffrom, *fto;

  ffrom = search_folder(from);
  fto = search_folder(to);
  if (!ffrom)
  {
    info(2, "Error searching folder '%s'\n", from);
    return -1;
  }

  if (!fto)
    return -2;

  add_mail(fto, mail->mail);
  delete_mail(ffrom, mail);
  return 0;
}

// delete the email pointed by the second argoument in the folder fold
// providing to free the memory associated. return the next mail to be
// selected if one
struct list *
delete_mail(struct folders *fold, struct list *mail)
{
  struct list *ret = NULL;

  // the first of the list
  if (!mail->prev)
  {
    fold->head = mail->next;
    ret = fold->head;
  }
  // the last one but not alone
  else if (!mail->next)
  {
    mail->prev->next = NULL;
    fold->tail = mail->prev;
    ret = mail->prev;
  }
  // a middleman
  else
  {
    mail->prev->next = mail->next;
    mail->next->prev = mail->prev;
    ret = mail->next;
  }
  if (mail->mail->type & NEW)
    fold->nnew--;
  fold->nall--;

  ranza(mail->mail->uidl);
  free_mail(mail->mail);

  // fold->nall--;
  return ret;
}

// free all memory allocated for mail
void
free_mail(struct email *mail)
{
  byte i;

  // free the destination
  for (i = 0; i < mail->nto; i++)
    free(mail->to[i]);

  if (mail->to)
    free(mail->to);

  /*
   * TOFIX if (mail->bcc) { for (i = 0; i < mail->nbcc; i++)
   * free(mail->bcc[i]); free(mail->bcc); } 
   */

  // if (mail->bcc)
  // free(mail->bcc);

  // the subject
  if (mail->subject)
    free(mail->subject);

  // the from
  if (mail->from)
    free(mail->from);

  // the date
  // if (mail->date)
  // free(mail->date);
  logg("prima del for\n");
  // free the headers
  i = 0;

  if (mail->header)
  {
    while (i > mail->head_lines)
    {
      free(mail->header[i]);
      i++;
    }
    free(mail->header);
  }
  // free the attachment
  if (mail->nattach)
  {
    for (i = 0; i < mail->nattach; i++)
    {
      if (mail->attach[i]->name)
        free(mail->attach[i]->name);
      // if (mail->attach[i]->data)
      // free(mail->attach[i]->data);
      free(mail->attach[i]);
    }
  }
  // free the messageid
  if (mail->messageid)
    free(mail->messageid);
  // free the inreplyto
  if (mail->inreplyto)
    free(mail->inreplyto);
  // free the body
  if (mail->body)
  {
    for (i = 0; i < mail->body_lines; i++)
      free(mail->body[i]);
    free(mail->body);
  }
  free(mail);
}


/*
 * Convert i to a baseX character string and store it in b. (from nail-10.6)
 * The caller has to ensure that the size of b is sufficient.
 */
char *
itostr(base, i, b)
     unsigned base, i;
     char *b;
{
  char *p, *q, c;

  p = b;
  while (i != 0)
  {
    *p++ = basetable[i % base];
    i /= base;
  }
  *p-- = '\0';
  q = b;
  while (p >= q)
  {
    c = *q;
    *q++ = *p;
    *p-- = c;
  }
  return b;
}



/*
 * Create a Message-Id: header field. (from nail-10.6)
 * Use either the host name or the host part of the from address.
 */

// returned pointer has to be freed
char *
message_id()
{
  static unsigned long msgc;
  time_t t;
  int rd;
  long randbuf;
  char datestr[sizeof(time_t) * 2 + 1];
  char pidstr[sizeof(pid_t) * 2 + 1];
  char countstr[sizeof msgc * 2 + 1];
  char randstr[sizeof randbuf * 2 + 1];
  char buff[BUFFER_REAL];

  msgc++;
  time(&t);
  rd = open("/dev/random", O_RDONLY);
  if (rd != -1)
  {
    if (read(rd, &randbuf, sizeof randbuf) != sizeof randbuf)
    {
      srand((unsigned) (msgc * t));
      randbuf = (long) rand();
    }
    close(rd);
  } else
  {
    srand((unsigned) (msgc * t));
    randbuf = (long) rand();
  }
  itostr(16, (unsigned) t, datestr);
  itostr(36, (unsigned) getpid(), pidstr);
  itostr(36, (unsigned) msgc, countstr);
  itostr(36, (unsigned) randbuf, randstr);

  snprintf(buff, BUFFER_LEN, "%s.p4RaN0y.%s%.5s%s%.5s@%s",
           datestr, VERSION, pidstr, countstr, randstr, localhost);

  return strdup(buff);

}


void
sort_mails(struct folders *fold, ushort mode)
{
  struct list *n = NULL, *tmp = fold->head, *one, *c;


  while (tmp)
  {
    if (!n)
    {
      n = (struct list *) smalloc(sizeof(struct list));
      n->mail = tmp->mail;
      n->next = NULL;
      n->prev = NULL;

    } else
    {
      c = n;
      while (c)
      {
        if (strcmp((char *) tmp->mail + mode, (char *) c->mail + mode) > 0)
        {
          one = (struct list *) malloc(sizeof(struct list));
          one->mail = tmp->mail;
          one->next = c;
          if (c->prev)
            c->prev->next = one;
          else
            n = one;
          one->prev = c->prev;
          c->prev = one;

          break;
        } else if (!c->next)
        {
          one = (struct list *) malloc(sizeof(struct list));
          one->mail = tmp->mail;
          one->next = NULL;
          one->prev = c;
          c->next = one;
          break;

        }
        c = c->next;
      }

    }
    tmp = tmp->next;
    if (tmp)
      free(tmp->prev);
  }
  fold->head = n;

}


char *
nexttoken(char *cp)
{

  while (!strchr("\n\t=\" \0", *cp))
    cp++;

  while (strchr("\n\t=\" ", *cp))
    cp++;

  return cp;
}

// bakunin's search elite functions
int
search_cmp_fragment(char *a, char *b)
{
  int l1, l2;

  l1 = strlen(a);
  l2 = strlen(b);

  while (l2 <= l1)
  {

    l1 = strlen(a);

    if (!strncasecmp(a, b, l2))
      return 1;

    a++;
  }

  return 0;
}

int
search_cmp(char *a, char *b)
{
  char dump[1024];
  char part[1024];
  int k = 0;
  int i;
  int len = strlen(b);
  int ret = 1;

  strncpy(dump, b, sizeof(dump));
  dump[len] = 0;

  for (i = 0; i < len; i++)
  {
    if (dump[i] == ' ' || dump[i] == '\t' || dump[i] == '\v'
        || dump[i] == '\n')
    {

      if (i > k)
      {
        strncpy(part, dump + k, i - k);
        part[i - k] = 0;
        ret = search_cmp_fragment(a, part);
        if (!ret)
          break;
      }
      k = i + 1;
    }
  }

  if (i > k && ret > 0)
  {
    strncpy(part, dump + k, i - k);
    part[i - k] = 0;
    ret = search_cmp_fragment(a, part);
  }

  return ret;
}
