#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "../include.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

GtkWidget *
create_common_button_box(void *ok_handler, GtkWidget * window)
{
  GtkWidget *out_hbox, *label, *in_hbox, *cancel_button, *ok_button;

  out_hbox = gtk_hbox_new(TRUE, 0);

  label = gtk_label_new("");
  gtk_box_pack_start(GTK_BOX(out_hbox), label, FALSE, FALSE, 0);

  in_hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(out_hbox), in_hbox, TRUE, TRUE, 0);

  cancel_button = gtk_button_new_with_mnemonic(_("Cancel"));
  gtk_box_pack_start(GTK_BOX(in_hbox), cancel_button, TRUE, FALSE, 0);
  gtk_widget_set_size_request(cancel_button, 80, -1);

  ok_button = gtk_button_new_with_mnemonic(_("Ok"));
  gtk_box_pack_start(GTK_BOX(in_hbox), ok_button, TRUE, FALSE, 0);
  gtk_widget_set_size_request(ok_button, 80, -1);

  g_signal_connect((gpointer) cancel_button, "clicked",
                   G_CALLBACK(on_cancel_button_clicked), window);
  g_signal_connect((gpointer) ok_button, "clicked",
                   G_CALLBACK(ok_handler), NULL);

  return out_hbox;
}

GtkWidget *
create_smtp_table(void)
{
  GtkWidget *label;

  smtpauth_table = gtk_table_new(2, 2, FALSE);

  label = gtk_label_new(_(" :: User"));
  gtk_table_attach(GTK_TABLE(smtpauth_table), label, 1, 2, 0, 1,
                   (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  label = gtk_label_new(_(" :: Pass"));
  gtk_table_attach(GTK_TABLE(smtpauth_table), label, 1, 2, 1, 2,
                   (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  smtppass_entry = gtk_entry_new();
  gtk_table_attach(GTK_TABLE(smtpauth_table), smtppass_entry, 0, 1, 1, 2,
                   (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);

  smtpuser_entry = gtk_entry_new();
  gtk_table_attach(GTK_TABLE(smtpauth_table), smtpuser_entry, 0, 1, 0, 1,
                   (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);

  return smtpauth_table;
}

GtkWidget *
create_settings_window(int page)
{
  GtkWidget *settings_window;
  GtkWidget *settings_notebook;
  GtkWidget *table;
  GtkWidget *hbox, *vbox;
  GtkWidget *time_combobox;
  GtkWidget *smtpauth_table;
  GtkWidget *label;
  GtkWidget *out_hbox;
  GtkWidget *changedir_button;
  GtkWidget *goption_label, *goptapply_button, *goptcancel_button,
    *goptok_button;
  GtkWidget *hseparator;
  GtkWidget *smtpport_entry;
  GtkWidget *paradir_entry;
  GtkWidget *parapass_button;
  GtkWidget *clearsmtp_radiobutton;
  GtkWidget *smtpssl_radiobutton;
  GSList *clearsmtp_radiobutton_group = NULL;
  GtkWidget *scrolledwindow;
  GtkWidget *sign_tree;
  GtkWidget *signbutton_hbox;
  GtkWidget *addsign_button;
  GtkWidget *modsign_button;
  GtkWidget *delsign_button;
  GtkWidget *viewcontact_button;
  GtkWidget *delcontact_button;
  GtkWidget *addcontact_button;
  GtkWidget *write_button;
  GtkWidget *addgroup_button;
  GtkWidget *delgroup_button;
  GtkWidget *contact_buttons_table;
  GtkWidget *viewport;
  GtkTreeViewColumn *col;
  GtkCellRenderer *renderer;
  GtkTreeSelection *groups_sel, *contacts_sel, *filters_sel;

  GtkWidget *account_table;
  GtkWidget *acclistbutton_hbox;
  GtkWidget *addaccount_button;
  GtkWidget *modaccount_button;
  GtkWidget *delaccount_button;
  GtkWidget *accounts_label;
  GtkWidget *contacts_label;
  GtkWidget *filters_tree;
  GtkWidget *filteradd_button;
  GtkWidget *filtermod_button;
  GtkWidget *filterdel_button, *moveup_button, *movedown_button;

  GtkWidget *filters_label;
  GtkWidget *pgp_soon_label;
  GtkWidget *pgp_label;
  GtkWidget *security_soon_label;
  GtkWidget *security_label;

  struct dirent **themes;
  int i;
  char buff[BUFFER_REAL];

  GtkTreeViewColumn *account_col;
  GtkCellRenderer *account_rend;

  // char path[300];

  current_group = NULL;
  current_contact = NULL;
  current_filter = NULL;

  settings_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  // gtk_widget_set_size_request (settings_window, 470, 600);
  gtk_window_set_title(GTK_WINDOW(settings_window),
                       _(":: paranoy ::"));

  settings_notebook = gtk_notebook_new();
  gtk_widget_show(settings_notebook);
  gtk_container_add(GTK_CONTAINER(settings_window), settings_notebook);
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(settings_notebook), GTK_POS_RIGHT);


  table = gtk_table_new(16, 1, FALSE);
  gtk_widget_show(table);
  gtk_container_add(GTK_CONTAINER(settings_notebook), table);
  gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(settings_notebook), table,
                                     TRUE, TRUE, GTK_PACK_START);
  // gtk_notebook_set_menu_label_text (GTK_NOTEBOOK (settings_notebook),
  // table,
  // _("menulabel???"));

  label =
    gtk_label_new(_
                  ("<b>:: SMTP infos ::</b>\n\nEnter SMTP address, the one which will be used to send your\nmails\n[as \"smtp.yourprovider.com\"]"));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (0), 30, 15);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 30, 0);


  label = gtk_label_new(_("SMTP :: "));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  smtpserver_entry = gtk_entry_new();
  gtk_widget_show(smtpserver_entry);
  gtk_box_pack_start(GTK_BOX(hbox), smtpserver_entry, TRUE, TRUE, 0);

  if (smtpserver)
    gtk_entry_set_text(GTK_ENTRY(smtpserver_entry), smtpserver);

  label = gtk_label_new(_(" :: "));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  smtpport_entry = gtk_entry_new();
  gtk_widget_show(smtpport_entry);
  gtk_box_pack_start(GTK_BOX(hbox), smtpport_entry, FALSE, FALSE, 0);
  gtk_widget_set_size_request(smtpport_entry, 40, -1);

  label = gtk_label_new(_(" :: Port"));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  label = gtk_label_new(_("Select the type of connection"));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 30, 10);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 3, 4,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 30, 0);

  clearsmtp_radiobutton =
    gtk_radio_button_new_with_mnemonic(NULL, _("Clear   "));
  gtk_widget_show(clearsmtp_radiobutton);
  gtk_box_pack_start(GTK_BOX(hbox), clearsmtp_radiobutton, FALSE, FALSE, 0);
  gtk_radio_button_set_group(GTK_RADIO_BUTTON(clearsmtp_radiobutton),
                             clearsmtp_radiobutton_group);
  clearsmtp_radiobutton_group =
    gtk_radio_button_get_group(GTK_RADIO_BUTTON(clearsmtp_radiobutton));

  smtpssl_radiobutton =
    gtk_radio_button_new_with_mnemonic(NULL, _("SSL encrypted"));
  gtk_widget_show(smtpssl_radiobutton);
  gtk_box_pack_start(GTK_BOX(hbox), smtpssl_radiobutton, FALSE, FALSE, 0);
  gtk_radio_button_set_group(GTK_RADIO_BUTTON(smtpssl_radiobutton),
                             clearsmtp_radiobutton_group);
  clearsmtp_radiobutton_group =
    gtk_radio_button_get_group(GTK_RADIO_BUTTON(smtpssl_radiobutton));

  auth_checkbutton =
    gtk_check_button_new_with_mnemonic(_
                                       ("Check it if you need SMTP authentication"));
  gtk_widget_show(auth_checkbutton);
  gtk_table_attach(GTK_TABLE(table), auth_checkbutton, 0, 1, 4, 5,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 30, 10);

  smtpauth_table = create_smtp_table();
  gtk_table_attach(GTK_TABLE(table), smtpauth_table, 0, 1, 5, 6,
                   (GtkAttachOptions) (0),
                   (GtkAttachOptions) (GTK_FILL), 50, 0);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 6, 7,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 25, 15);

  label =
    gtk_label_new(_
                  ("<b>:: Paranoy's stuff ::</b>\n\nThis is the directory where paranoy will store your mails and\nconfiguration."));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 7, 8,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 30, 0);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 8, 9,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 30, 10);

  paradir_entry = gtk_entry_new();
  gtk_widget_show(paradir_entry);
  gtk_box_pack_start(GTK_BOX(hbox), paradir_entry, TRUE, TRUE, 0);

  changedir_button = gtk_button_new_with_mnemonic(_("Change directory"));
  gtk_widget_show(changedir_button);
  gtk_box_pack_start(GTK_BOX(hbox), changedir_button, FALSE, FALSE, 0);
  gtk_button_set_relief(GTK_BUTTON(changedir_button), GTK_RELIEF_NONE);

  parapass_button =
    gtk_button_new_with_mnemonic(_
                                 ("Click here to change paranoy's password"));
  gtk_widget_show(parapass_button);
  gtk_table_attach(GTK_TABLE(table), parapass_button, 0, 1, 9, 10,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 30, 0);
  gtk_button_set_relief(GTK_BUTTON(parapass_button), GTK_RELIEF_NONE);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 10, 11,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 25, 10);

  label = gtk_label_new(_("<b>:: Signatures ::</b>"));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 11, 12,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 30, 0);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_show(scrolledwindow);
  gtk_table_attach(GTK_TABLE(table), scrolledwindow, 0, 1, 12, 13,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 30, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwindow),
                                      GTK_SHADOW_IN);

  sign_tree = gtk_tree_view_new();
  gtk_widget_show(sign_tree);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), sign_tree);

  signbutton_hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(signbutton_hbox);
  gtk_table_attach(GTK_TABLE(table), signbutton_hbox, 0, 1, 13, 14,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 30, 0);

  addsign_button = gtk_button_new_with_mnemonic(_("Add signature"));
  gtk_widget_show(addsign_button);
  gtk_box_pack_start(GTK_BOX(signbutton_hbox), addsign_button, TRUE, TRUE, 0);

  modsign_button = gtk_button_new_with_mnemonic(_("Modify signature"));
  gtk_widget_show(modsign_button);
  gtk_box_pack_start(GTK_BOX(signbutton_hbox), modsign_button, TRUE, TRUE, 0);

  delsign_button = gtk_button_new_with_mnemonic(_("Delete signture"));
  gtk_widget_show(delsign_button);
  gtk_box_pack_start(GTK_BOX(signbutton_hbox), delsign_button, TRUE, TRUE, 0);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 14, 15,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);

  out_hbox =
    create_common_button_box(on_settingok_button_clicked, settings_window);
  gtk_widget_show_all(out_hbox);
  gtk_table_attach(GTK_TABLE(table), out_hbox, 0, 1, 15, 16,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);

  // ---------------

  goptapply_button = gtk_button_new_with_mnemonic(_("Ok"));

  goptcancel_button = gtk_button_new_with_mnemonic(_("Ok"));

  goptok_button = gtk_button_new_with_mnemonic(_("Ok"));
  gtk_widget_show(goptok_button);


  // gtk_widget_set_size_request (ok_button, 60, -1);



  // --------------


  goption_label = gtk_label_new(_("General"));
  gtk_widget_show(goption_label);
  gtk_notebook_set_tab_label(GTK_NOTEBOOK(settings_notebook),
                             gtk_notebook_get_nth_page(GTK_NOTEBOOK
                                                       (settings_notebook),
                                                       0), goption_label);
  gtk_label_set_justify(GTK_LABEL(goption_label), GTK_JUSTIFY_CENTER);

  account_table = gtk_table_new(8, 1, FALSE);
  gtk_widget_show(account_table);
  gtk_container_add(GTK_CONTAINER(settings_notebook), account_table);
  gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(settings_notebook),
                                     account_table, TRUE, TRUE,
                                     GTK_PACK_START);

  label =
    gtk_label_new(_
                  ("<b>:: Your accounts ::</b>\n\nHere you can view and manage your mail accounts"));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(account_table), label, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (0), 30, 15);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_show(scrolledwindow);
  gtk_table_attach(GTK_TABLE(account_table), scrolledwindow, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 30, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwindow),
                                      GTK_SHADOW_IN);

  /*
   * viewport = gtk_viewport_new (NULL, NULL); gtk_widget_show (viewport);
   * gtk_container_add (GTK_CONTAINER (scrolledwindow), viewport);
   */

  account_store = gtk_list_store_new(ACC_NCOL, G_TYPE_STRING, G_TYPE_STRING);

  print_accounts();

  account_list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(account_store));
  account_rend = gtk_cell_renderer_text_new();

  account_col =
    gtk_tree_view_column_new_with_attributes("Name", account_rend, "text",
                                             ACC_NAME, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(account_list), account_col);
  gtk_tree_view_column_set_min_width(GTK_TREE_VIEW_COLUMN(account_col), 100);

  account_col =
    gtk_tree_view_column_new_with_attributes("Email", account_rend, "text",
                                             ACC_EMAIL, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(account_list), account_col);

  gtk_container_add(GTK_CONTAINER(scrolledwindow), account_list);
  gtk_widget_show(account_list);


  acclistbutton_hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(acclistbutton_hbox);
  gtk_table_attach(GTK_TABLE(account_table), acclistbutton_hbox, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 30, 0);

  addaccount_button = gtk_button_new_with_mnemonic(_("Add account"));
  gtk_widget_show(addaccount_button);
  gtk_box_pack_start(GTK_BOX(acclistbutton_hbox), addaccount_button, TRUE,
                     TRUE, 0);
  // gtk_button_set_relief (GTK_BUTTON (addaccount_button), GTK_RELIEF_NONE);

  modaccount_button = gtk_button_new_with_mnemonic(_("Modify account"));
  gtk_widget_show(modaccount_button);
  gtk_box_pack_start(GTK_BOX(acclistbutton_hbox), modaccount_button, TRUE,
                     TRUE, 0);
  // gtk_button_set_relief (GTK_BUTTON (modaccount_button), GTK_RELIEF_NONE);

  delaccount_button = gtk_button_new_with_mnemonic(_("Delete account"));
  gtk_widget_show(delaccount_button);
  gtk_box_pack_start(GTK_BOX(acclistbutton_hbox), delaccount_button, TRUE,
                     TRUE, 0);
  // gtk_button_set_relief (GTK_BUTTON (delaccount_button), GTK_RELIEF_NONE);
  // gtk_button_set_focus_on_click (GTK_BUTTON (delaccount_button), FALSE);

  hbox =
    create_common_button_box(on_settingok_button_clicked, settings_window);
  gtk_widget_show_all(hbox);
  gtk_table_attach(GTK_TABLE(account_table), hbox, 0, 1, 7, 8,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(account_table), hseparator, 0, 1, 3, 4,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 25, 15);

  label =
    gtk_label_new(_
                  ("<b>:: Mail check time ::</b>\n\nParanoy checks automatically new mails for your accounts. \nHere you can set the frequency or disable this job."));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(account_table), label, 0, 1, 4, 5,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 30, 0);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  gtk_table_attach(GTK_TABLE(account_table), hbox, 0, 1, 5, 6,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 35, 10);

  label = gtk_label_new(_("Check time :: "));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  time_combobox = gtk_combo_box_new_text();
  gtk_widget_show(time_combobox);
  gtk_box_pack_start(GTK_BOX(hbox), time_combobox, FALSE, TRUE, 0);
  gtk_combo_box_append_text(GTK_COMBO_BOX(time_combobox), _("none"));
  gtk_combo_box_append_text(GTK_COMBO_BOX(time_combobox), _("one minute"));
  gtk_combo_box_append_text(GTK_COMBO_BOX(time_combobox), _("ten minutes"));
  gtk_combo_box_append_text(GTK_COMBO_BOX(time_combobox), _("half hour"));
  gtk_combo_box_append_text(GTK_COMBO_BOX(time_combobox), _("one hour"));
  gtk_combo_box_set_active(GTK_COMBO_BOX(time_combobox), 0);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(account_table), hseparator, 0, 1, 6, 7,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);


  accounts_label = gtk_label_new(_("Accounts"));
  gtk_widget_show(accounts_label);
  gtk_notebook_set_tab_label(GTK_NOTEBOOK(settings_notebook),
                             gtk_notebook_get_nth_page(GTK_NOTEBOOK
                                                       (settings_notebook),
                                                       1), accounts_label);

  table = gtk_table_new(8, 1, FALSE);
  gtk_widget_show(table);
  gtk_container_add(GTK_CONTAINER(settings_notebook), table);
  gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(settings_notebook), table,
                                     TRUE, TRUE, GTK_PACK_START);

  label =
    gtk_label_new(_
                  ("<b>:: Address book ::</b>\n\nHere you can view and manage your contacts"));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (0), 30, 15);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  hbox = gtk_hbox_new(TRUE, 0);
  gtk_widget_show(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 30, 0);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);

  newgroup_vbox = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_end((GtkBox *) vbox, newgroup_vbox, FALSE, FALSE, 0);

  label = gtk_label_new(_("Insert name of the group"));
  gtk_box_pack_start((GtkBox *) newgroup_vbox, label, FALSE, TRUE, 0);

  newgroup_entry = gtk_entry_new();
  gtk_box_pack_start((GtkBox *) newgroup_vbox, newgroup_entry, TRUE, TRUE, 0);


  scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_show(scrolledwindow);
  gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  viewport = gtk_viewport_new(NULL, NULL);
  gtk_widget_show(viewport);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), viewport);
  gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_IN);



  // aggiungo i gruppi

  groups_list = gtk_list_store_new(GROUP_NCOL, G_TYPE_STRING, G_TYPE_POINTER);

  print_contacts_group();

  groups_tree = gtk_tree_view_new_with_model((GtkTreeModel *) groups_list);
  renderer = gtk_cell_renderer_text_new();

  col =
    gtk_tree_view_column_new_with_attributes("Groups", renderer, "text",
                                             GROUP_NAME, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(groups_tree), col);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(col), TRUE);

  gtk_widget_show(groups_tree);
  gtk_container_add((GtkContainer *) viewport, groups_tree);
  // gtk_box_pack_start((GtkBox *)vbox, groups_tree, TRUE, TRUE, 0);
  groups_sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(groups_tree));



  contact_buttons_table = gtk_table_new(7, 1, FALSE);
  gtk_widget_show(contact_buttons_table);
  gtk_box_pack_start(GTK_BOX(hbox), contact_buttons_table, TRUE, TRUE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(contact_buttons_table), 10);

  viewcontact_button = gtk_button_new_with_mnemonic(_("View contact"));
  gtk_widget_show(viewcontact_button);
  gtk_table_attach(GTK_TABLE(contact_buttons_table), viewcontact_button, 0, 1,
                   5, 6, (GtkAttachOptions) (GTK_EXPAND),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_size_request(viewcontact_button, 100, -1);

  delcontact_button = gtk_button_new_with_mnemonic(_("Delete contact"));
  gtk_widget_show(delcontact_button);
  gtk_table_attach(GTK_TABLE(contact_buttons_table), delcontact_button, 0, 1,
                   4, 5, (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0,
                   0);
  gtk_widget_set_size_request(delcontact_button, 100, -1);

  addcontact_button = gtk_button_new_with_mnemonic(_("Add contact"));
  gtk_widget_show(addcontact_button);
  gtk_table_attach(GTK_TABLE(contact_buttons_table), addcontact_button, 0, 1,
                   3, 4, (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0,
                   0);
  gtk_widget_set_size_request(addcontact_button, 100, -1);

  delgroup_button = gtk_button_new_with_mnemonic(_("Delete group"));
  gtk_widget_show(delgroup_button);
  gtk_table_attach(GTK_TABLE(contact_buttons_table), delgroup_button, 0, 1, 1,
                   2, (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_size_request(delgroup_button, 100, -1);

  write_button = gtk_button_new_with_mnemonic(_("Write"));
  gtk_widget_show(write_button);
  gtk_table_attach(GTK_TABLE(contact_buttons_table), write_button, 0, 1, 6, 7,
                   (GtkAttachOptions) (GTK_EXPAND),
                   (GtkAttachOptions) (0), 0, 10);
  gtk_widget_set_size_request(write_button, 100, -1);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(contact_buttons_table), hseparator, 0, 1, 2, 3,
                   (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 5);
  gtk_widget_set_size_request(hseparator, 110, -1);

  addgroup_button = gtk_button_new_with_mnemonic(_("Add group"));
  gtk_widget_show(addgroup_button);
  gtk_table_attach(GTK_TABLE(contact_buttons_table), addgroup_button, 0, 1, 0,
                   1, (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_size_request(addgroup_button, 100, -1);

  scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_show(scrolledwindow);
  gtk_table_attach(GTK_TABLE(table), scrolledwindow, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 30, 10);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  viewport = gtk_viewport_new(NULL, NULL);
  gtk_widget_show(viewport);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), viewport);
  gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_IN);


  // aggiungo i contacts

  contacts_list =
    gtk_list_store_new(CON_NCOL, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
                       G_TYPE_POINTER);

  contacts_tree =
    gtk_tree_view_new_with_model((GtkTreeModel *) contacts_list);

  col =
    gtk_tree_view_column_new_with_attributes("Name", renderer, "text",
                                             CON_NAME, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(contacts_tree), col);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(col), TRUE);
  gtk_tree_view_column_set_min_width(GTK_TREE_VIEW_COLUMN(col), 100);

  col =
    gtk_tree_view_column_new_with_attributes("Email", renderer, "text",
                                             CON_EMAIL, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(contacts_tree), col);
  gtk_tree_view_column_set_expand(GTK_TREE_VIEW_COLUMN(col), TRUE);

  // gtk_tree_view_column_set_max_width(GTK_TREE_VIEW_COLUMN(col), 80);

  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(col), TRUE);

  col =
    gtk_tree_view_column_new_with_attributes("PGP", renderer, "text", CON_KEY,
                                             NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(contacts_tree), col);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(col), TRUE);


  gtk_widget_show(contacts_tree);
  gtk_container_add((GtkContainer *) viewport, contacts_tree);

  contacts_sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(contacts_tree));

  label =
    gtk_label_new(_
                  ("<b>:: Search ::</b>\n\nType here to serch a contacts in your address book."));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 30, 10);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 5, 6,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 30, 0);

  label = gtk_label_new(_("Look for :: "));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  search_contact_entry = gtk_entry_new();
  gtk_widget_show(search_contact_entry);
  gtk_box_pack_start(GTK_BOX(hbox), search_contact_entry, TRUE, TRUE, 0);

  label = gtk_label_new(_(" in "));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  search_field_combobox = gtk_combo_box_new_text();
  gtk_widget_show(search_field_combobox);
  gtk_box_pack_start(GTK_BOX(hbox), search_field_combobox, TRUE, TRUE, 0);
  gtk_combo_box_append_text(GTK_COMBO_BOX(search_field_combobox), _("name"));
  gtk_combo_box_append_text(GTK_COMBO_BOX(search_field_combobox), _("email"));
  gtk_combo_box_append_text(GTK_COMBO_BOX(search_field_combobox), _("both"));
  gtk_combo_box_set_active(GTK_COMBO_BOX(search_field_combobox), 0);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 3, 4,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 25, 5);

  hbox =
    create_common_button_box(on_settingok_button_clicked, settings_window);
  gtk_widget_show_all(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 7, 8,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 6, 7,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);



  contacts_label = gtk_label_new(_("Contacts"));
  gtk_widget_show(contacts_label);
  gtk_notebook_set_tab_label(GTK_NOTEBOOK(settings_notebook),
                             gtk_notebook_get_nth_page(GTK_NOTEBOOK
                                                       (settings_notebook),
                                                       2), contacts_label);
  gtk_label_set_justify(GTK_LABEL(contacts_label), GTK_JUSTIFY_CENTER);

  table = gtk_table_new(5, 1, FALSE);
  gtk_widget_show(table);
  gtk_container_add(GTK_CONTAINER(settings_notebook), table);
  gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(settings_notebook), table,
                                     TRUE, TRUE, GTK_PACK_START);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 3, 4,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);

  label =
    gtk_label_new(_
                  ("<b>:: Filters ::</b>\n\nHere you can view and manage filters for you incoming mails"));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 30, 15);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  hbox = gtk_hbox_new(TRUE, 0);
  gtk_widget_show(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 30, 0);

  scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_show(scrolledwindow);
  gtk_box_pack_start(GTK_BOX(hbox), scrolledwindow, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  viewport = gtk_viewport_new(NULL, NULL);
  gtk_widget_show(viewport);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), viewport);



  filters_list =
    gtk_list_store_new(FILTER_NCOL, G_TYPE_STRING, G_TYPE_POINTER);


  filters_tree = gtk_tree_view_new_with_model((GtkTreeModel *) filters_list);
  gtk_widget_show(filters_tree);
  renderer = gtk_cell_renderer_text_new();

  col =
    gtk_tree_view_column_new_with_attributes("Filters", renderer, "text",
                                             FILTER_NAME, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(filters_tree), col);
  gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(col), TRUE);

  filters_sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(groups_tree));


  gtk_container_add(GTK_CONTAINER(viewport), filters_tree);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);

  filteradd_button = gtk_button_new_with_mnemonic(_("Add filter"));
  gtk_widget_show(filteradd_button);
  gtk_box_pack_start(GTK_BOX(vbox), filteradd_button, FALSE, FALSE, 0);
  gtk_widget_set_size_request(filteradd_button, 100, -1);

  filterdel_button = gtk_button_new_with_mnemonic(_("Del filter"));
  gtk_widget_show(filterdel_button);
  gtk_box_pack_start(GTK_BOX(vbox), filterdel_button, FALSE, FALSE, 0);
  gtk_widget_set_size_request(filterdel_button, 100, -1);

  filtermod_button = gtk_button_new_with_mnemonic(_("View filter"));
  gtk_widget_show(filtermod_button);
  gtk_box_pack_start(GTK_BOX(vbox), filtermod_button, FALSE, FALSE, 0);
  gtk_widget_set_size_request(filtermod_button, 100, -1);

  movedown_button = gtk_button_new_with_mnemonic(_("Move down"));
  gtk_widget_show(movedown_button);
  gtk_box_pack_end(GTK_BOX(vbox), movedown_button, FALSE, FALSE, 10);
  gtk_widget_set_size_request(movedown_button, 100, -1);

  moveup_button = gtk_button_new_with_mnemonic(_("Move up"));
  gtk_widget_show(moveup_button);
  gtk_box_pack_end(GTK_BOX(vbox), moveup_button, FALSE, FALSE, 0);
  gtk_widget_set_size_request(moveup_button, 100, -1);



  out_hbox =
    create_common_button_box(on_settingok_button_clicked, settings_window);
  gtk_widget_show_all(out_hbox);
  gtk_table_attach(GTK_TABLE(table), out_hbox, 0, 1, 4, 5,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);

  label = gtk_label_new("");
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);


  filters_label = gtk_label_new(_("Filters"));
  gtk_widget_show(filters_label);
  gtk_notebook_set_tab_label(GTK_NOTEBOOK(settings_notebook),
                             gtk_notebook_get_nth_page(GTK_NOTEBOOK
                                                       (settings_notebook),
                                                       3), filters_label);

  table = gtk_table_new(3, 1, FALSE);
  gtk_widget_show(table);
  gtk_container_add(GTK_CONTAINER(settings_notebook), table);
  gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(settings_notebook), table,
                                     TRUE, TRUE, GTK_PACK_START);



  label = gtk_label_new(_("Coming soon..."));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_EXPAND), 30, 15);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);

  hbox =
    create_common_button_box(on_settingok_button_clicked, settings_window);
  gtk_widget_show_all(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);


  pgp_label = gtk_label_new(_("PGP"));
  gtk_widget_show(pgp_label);
  gtk_notebook_set_tab_label(GTK_NOTEBOOK(settings_notebook),
                             gtk_notebook_get_nth_page(GTK_NOTEBOOK
                                                       (settings_notebook),
                                                       4), pgp_label);

  table = gtk_table_new(3, 1, FALSE);
  gtk_widget_show(table);
  gtk_container_add(GTK_CONTAINER(settings_notebook), table);
  gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(settings_notebook), table,
                                     TRUE, TRUE, GTK_PACK_START);



  label = gtk_label_new(_("Coming soon..."));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_EXPAND), 30, 15);
  gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(settings_notebook),
                                     label, TRUE, TRUE, GTK_PACK_START);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);

  hbox =
    create_common_button_box(on_settingok_button_clicked, settings_window);
  gtk_widget_show_all(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);



  security_label = gtk_label_new(_("Security"));
  gtk_widget_show(security_label);
  gtk_notebook_set_tab_label(GTK_NOTEBOOK(settings_notebook),
                             gtk_notebook_get_nth_page(GTK_NOTEBOOK
                                                       (settings_notebook),
                                                       5), security_label);

  table = gtk_table_new(3, 1, FALSE);
  gtk_widget_show(table);
  gtk_container_add(GTK_CONTAINER(settings_notebook), table);
  gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(settings_notebook), table,
                                     TRUE, TRUE, GTK_PACK_START);



  label = gtk_label_new(_("Coming soon..."));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_EXPAND), 30, 15);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);

  hbox =
    create_common_button_box(on_settingok_button_clicked, settings_window);
  gtk_widget_show_all(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);

  label = gtk_label_new(_("Anonimity"));
  gtk_widget_show(label);
  gtk_notebook_set_tab_label(GTK_NOTEBOOK(settings_notebook),
                             gtk_notebook_get_nth_page(GTK_NOTEBOOK
                                                       (settings_notebook),
                                                       6), label);
  printf("alla gui\n");

  table = gtk_table_new(6, 1, FALSE);
  gtk_widget_show(table);
  gtk_container_add(GTK_CONTAINER(settings_notebook), table);
  gtk_notebook_set_tab_label_packing(GTK_NOTEBOOK(settings_notebook), table,
                                     TRUE, TRUE, GTK_PACK_START);

  label =
    gtk_label_new(_
                  ("<b>:: Themes ::</b>\n\nHere you can change paranoy theme. The default folder \nfor themes is '$CHOSEN_PATH/.paranoy/Themes/'."));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 30, 15);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  // gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 5, 10);

  label = gtk_label_new(_("Paranoy's themes :: "));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  themes_combobox = gtk_combo_box_new_text();
  gtk_widget_show(themes_combobox);
  gtk_box_pack_start(GTK_BOX(hbox), themes_combobox, FALSE, TRUE, 0);

  printf("prima dello scandir\n");

  snprintf(buff, BUFFER_LEN, "%s/Themes", MAILDIR);
  i = scandir(buff, &themes, NULL, NULL);
  printf("i e' -- %d\n", i);
  if (i >= 0)
  {
    while (i--)
    {
      sprintf(buff, "%s", themes[i]->d_name);
      if ((strcmp(buff, ".")) && (strcmp(buff, "..")))
        gtk_combo_box_append_text(GTK_COMBO_BOX(themes_combobox), _(buff));
      free(themes[i]);
    }
    free(themes);
  }

  gtk_combo_box_set_active((GtkComboBox *) themes_combobox, 0);




  label = gtk_label_new(_(""));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_EXPAND), 30, 15);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);


  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 4, 5,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);

  hbox =
    create_common_button_box(on_settingok_button_clicked, settings_window);
  gtk_widget_show_all(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 5, 6,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);



  label = gtk_label_new(_("Gui aspect"));
  gtk_widget_show(label);
  gtk_notebook_set_tab_label(GTK_NOTEBOOK(settings_notebook),
                             gtk_notebook_get_nth_page(GTK_NOTEBOOK
                                                       (settings_notebook),
                                                       7), label);


  gtk_notebook_set_current_page(GTK_NOTEBOOK(settings_notebook), page);

  g_signal_connect((gpointer) settings_window, "destroy_event",
                   G_CALLBACK(on_settings_window_destroy_event), NULL);
  g_signal_connect((gpointer) parapass_button, "clicked",
                   G_CALLBACK(on_parapass_button_clicked), NULL);
  g_signal_connect((gpointer) auth_checkbutton, "toggled",
                   G_CALLBACK(toggle_smtp_auth_info), NULL);
  g_signal_connect((gpointer) groups_sel, "changed",
                   G_CALLBACK(on_group_changed), NULL);
  g_signal_connect((gpointer) contacts_sel, "changed",
                   G_CALLBACK(on_contact_changed), NULL);
  g_signal_connect((gpointer) newgroup_entry, "activate",
                   G_CALLBACK(on_newgroup_done), NULL);
  g_signal_connect((gpointer) delgroup_button, "clicked",
                   G_CALLBACK(on_delgroup_button_clicked), NULL);
  g_signal_connect((gpointer) modsign_button, "clicked",
                   G_CALLBACK(on_modsign_button_clicked), NULL);
  g_signal_connect((gpointer) delsign_button, "clicked",
                   G_CALLBACK(on_delsign_button_clicked), NULL);
  g_signal_connect((gpointer) goptapply_button, "clicked",
                   G_CALLBACK(on_settingapply_button_clicked), NULL);
  g_signal_connect((gpointer) goptcancel_button, "clicked",
                   G_CALLBACK(on_settingcancel_button_clicked), NULL);
  g_signal_connect((gpointer) goptok_button, "clicked",
                   G_CALLBACK(on_settingok_button_clicked), NULL);
  g_signal_connect((gpointer) addaccount_button, "clicked",
                   G_CALLBACK(on_addaccount_button_clicked), NULL);
  g_signal_connect((gpointer) modaccount_button, "clicked",
                   G_CALLBACK(on_modaccount_button_clicked), NULL);
  g_signal_connect((gpointer) delaccount_button, "clicked",
                   G_CALLBACK(on_delacc_button_clicked), NULL);
  g_signal_connect((gpointer) addgroup_button, "clicked",
                   G_CALLBACK(on_new_group_activate), NULL);
  g_signal_connect((gpointer) addcontact_button, "clicked",
                   G_CALLBACK(on_contactadd_button_clicked), NULL);
  g_signal_connect((gpointer) write_button, "clicked",
                   G_CALLBACK(on_compose_button_clicked), NULL);
  g_signal_connect((gpointer) viewcontact_button, "clicked",
                   G_CALLBACK(on_contactmod_button_clicked), NULL);
  g_signal_connect((gpointer) delcontact_button, "clicked",
                   G_CALLBACK(on_contactdel_button_clicked), NULL);
  g_signal_connect((gpointer) filteradd_button, "clicked",
                   G_CALLBACK(on_filteradd_button_clicked), NULL);
  g_signal_connect((gpointer) filtermod_button, "clicked",
                   G_CALLBACK(on_filtermod_button_clicked), NULL);
  g_signal_connect((gpointer) filterdel_button, "clicked",
                   G_CALLBACK(on_filterddel_button_clicked), NULL);

  return settings_window;
}



GtkWidget *
create_parapass_window(void)
{
  GtkWidget *parapass_window;
  GtkWidget *parapass_vbox;
  GtkWidget *parapass_table;
  GtkWidget *oldpass_entry;
  GtkWidget *newpass_label;
  GtkWidget *olpass_label;
  GtkWidget *newpass_entry;
  GtkWidget *confirmpass_label;
  GtkWidget *confirmpass_entry;
  GtkWidget *label195;
  GtkWidget *label196;
  GtkWidget *label197;
  GtkWidget *label198;
  GtkWidget *label199;
  GtkWidget *label200;
  GtkWidget *parapass_hseparator;
  GtkWidget *parapassbutton_hbox;
  GtkWidget *passcancel_button;
  GtkWidget *paraok_button;

  parapass_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(parapass_window),
                       _("-= change password =-"));
  gtk_window_set_destroy_with_parent(GTK_WINDOW(parapass_window), TRUE);

  parapass_vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(parapass_vbox);
  gtk_container_add(GTK_CONTAINER(parapass_window), parapass_vbox);

  parapass_table = gtk_table_new(3, 4, FALSE);
  gtk_widget_show(parapass_table);
  gtk_box_pack_start(GTK_BOX(parapass_vbox), parapass_table, FALSE, TRUE, 0);

  oldpass_entry = gtk_entry_new();
  gtk_widget_show(oldpass_entry);
  gtk_table_attach(GTK_TABLE(parapass_table), oldpass_entry, 2, 3, 0, 1,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_entry_set_visibility(GTK_ENTRY(oldpass_entry), FALSE);
  gtk_entry_set_activates_default(GTK_ENTRY(oldpass_entry), TRUE);

  newpass_label = gtk_label_new(_("New password:"));
  gtk_widget_show(newpass_label);
  gtk_table_attach(GTK_TABLE(parapass_table), newpass_label, 1, 2, 1, 2,
                   (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 5);
  gtk_misc_set_alignment(GTK_MISC(newpass_label), 0, 0.5);

  olpass_label = gtk_label_new(_("Old password:"));
  gtk_widget_show(olpass_label);
  gtk_table_attach(GTK_TABLE(parapass_table), olpass_label, 1, 2, 0, 1,
                   (GtkAttachOptions) (GTK_EXPAND),
                   (GtkAttachOptions) (0), 0, 15);
  gtk_misc_set_alignment(GTK_MISC(olpass_label), 0, 0.5);

  newpass_entry = gtk_entry_new();
  gtk_widget_show(newpass_entry);
  gtk_table_attach(GTK_TABLE(parapass_table), newpass_entry, 2, 3, 1, 2,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_entry_set_visibility(GTK_ENTRY(newpass_entry), FALSE);

  confirmpass_label = gtk_label_new(_("Confirm password:"));
  gtk_widget_show(confirmpass_label);
  gtk_table_attach(GTK_TABLE(parapass_table), confirmpass_label, 1, 2, 2, 3,
                   (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 5);
  gtk_misc_set_alignment(GTK_MISC(confirmpass_label), 0, 0.5);

  confirmpass_entry = gtk_entry_new();
  gtk_widget_show(confirmpass_entry);
  gtk_table_attach(GTK_TABLE(parapass_table), confirmpass_entry, 2, 3, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 0, 5);
  gtk_entry_set_visibility(GTK_ENTRY(confirmpass_entry), FALSE);

  label195 = gtk_label_new("");
  gtk_widget_show(label195);
  gtk_table_attach(GTK_TABLE(parapass_table), label195, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(label195), 0, 0.5);

  label196 = gtk_label_new("");
  gtk_widget_show(label196);
  gtk_table_attach(GTK_TABLE(parapass_table), label196, 3, 4, 0, 1,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(label196), 0, 0.5);

  label197 = gtk_label_new("");
  gtk_widget_show(label197);
  gtk_table_attach(GTK_TABLE(parapass_table), label197, 3, 4, 1, 2,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(label197), 0, 0.5);

  label198 = gtk_label_new("");
  gtk_widget_show(label198);
  gtk_table_attach(GTK_TABLE(parapass_table), label198, 3, 4, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(label198), 0, 0.5);

  label199 = gtk_label_new("");
  gtk_widget_show(label199);
  gtk_table_attach(GTK_TABLE(parapass_table), label199, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(label199), 0, 0.5);

  label200 = gtk_label_new("");
  gtk_widget_show(label200);
  gtk_table_attach(GTK_TABLE(parapass_table), label200, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(label200), 0, 0.5);

  parapass_hseparator = gtk_hseparator_new();
  gtk_widget_show(parapass_hseparator);
  gtk_box_pack_start(GTK_BOX(parapass_vbox), parapass_hseparator, FALSE, TRUE,
                     10);

  parapassbutton_hbox = gtk_hbox_new(TRUE, 0);
  gtk_widget_show(parapassbutton_hbox);
  gtk_box_pack_start(GTK_BOX(parapass_vbox), parapassbutton_hbox, FALSE, TRUE,
                     0);

  passcancel_button = gtk_button_new_with_mnemonic(_("Cancel"));
  gtk_widget_show(passcancel_button);
  gtk_box_pack_start(GTK_BOX(parapassbutton_hbox), passcancel_button, TRUE,
                     TRUE, 0);

  paraok_button = gtk_button_new_with_mnemonic(_("Ok"));
  gtk_widget_show(paraok_button);
  gtk_box_pack_start(GTK_BOX(parapassbutton_hbox), paraok_button, FALSE, TRUE,
                     0);

  g_signal_connect((gpointer) parapass_window, "destroy_event",
                   G_CALLBACK(on_parapass_window_destroy_event), NULL);
  g_signal_connect((gpointer) passcancel_button, "clicked",
                   G_CALLBACK(on_passcancel_button_clicked), NULL);
  g_signal_connect((gpointer) paraok_button, "clicked",
                   G_CALLBACK(on_paraok_button_clicked), NULL);

  return parapass_window;
}

GtkWidget *
create_accounts_window(void)
{
  GtkWidget *accounts_window;
  GtkWidget *main_table;
  GtkWidget *emailnick_hbox;
  GtkWidget *emailnick_label;
  GtkWidget *emailadd_hbox;
  GtkWidget *emailadd_label;
  GtkWidget *hseparator;
  GtkWidget *popport_hbox;
  GtkWidget *popport_label;
  GtkWidget *popuser_hbox;
  GtkWidget *popuser_label;
  GtkWidget *poppass_hbox;
  GtkWidget *poppass_label;
  GtkWidget *popserver_hbox;
  GtkWidget *popserver_label;
  GtkWidget *popproto_hbox;
  GtkWidget *popproto_label;
  GtkWidget *buttons_hbox;
  GtkWidget *cancel_button;
  GtkWidget *ok_button;

  accounts_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(accounts_window), _(":: account ::"));
  gtk_window_set_position(GTK_WINDOW(accounts_window), GTK_WIN_POS_CENTER);

  main_table = gtk_table_new(10, 1, FALSE);
  gtk_widget_show(main_table);
  gtk_container_add(GTK_CONTAINER(accounts_window), main_table);

  emailnick_hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(emailnick_hbox);
  gtk_table_attach(GTK_TABLE(main_table), emailnick_hbox, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 30, 15);

  emailnick_label = gtk_label_new(_("Email name  ::    "));
  gtk_widget_show(emailnick_label);
  gtk_box_pack_start(GTK_BOX(emailnick_hbox), emailnick_label, FALSE, FALSE,
                     0);

  emailnick_entry = gtk_entry_new();
  gtk_widget_show(emailnick_entry);
  gtk_box_pack_start(GTK_BOX(emailnick_hbox), emailnick_entry, FALSE, TRUE,
                     0);
  gtk_entry_set_text(GTK_ENTRY(emailnick_entry), _(" "));

  emailadd_hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(emailadd_hbox);
  gtk_table_attach(GTK_TABLE(main_table), emailadd_hbox, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 30, 0);

  emailadd_label = gtk_label_new(_("Email address  :: "));
  gtk_widget_show(emailadd_label);
  gtk_box_pack_start(GTK_BOX(emailadd_hbox), emailadd_label, FALSE, FALSE, 0);

  emailadd_entry = gtk_entry_new();
  gtk_widget_show(emailadd_entry);
  gtk_box_pack_start(GTK_BOX(emailadd_hbox), emailadd_entry, FALSE, TRUE, 0);
  gtk_widget_set_size_request(emailadd_entry, 300, -1);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(main_table), hseparator, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 80, 15);

  popport_hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(popport_hbox);
  gtk_table_attach(GTK_TABLE(main_table), popport_hbox, 0, 1, 6, 7,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 30, 0);

  popport_label = gtk_label_new(_("on port   ::            "));
  gtk_widget_show(popport_label);
  gtk_box_pack_start(GTK_BOX(popport_hbox), popport_label, FALSE, FALSE, 0);
  gtk_misc_set_alignment(GTK_MISC(popport_label), 0, 0.5);

  popport_entry = gtk_entry_new();
  gtk_widget_show(popport_entry);
  gtk_box_pack_start(GTK_BOX(popport_hbox), popport_entry, FALSE, TRUE, 0);
  gtk_widget_set_size_request(popport_entry, 40, -1);

  popuser_hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(popuser_hbox);
  gtk_table_attach(GTK_TABLE(main_table), popuser_hbox, 0, 1, 3, 4,
                   (GtkAttachOptions) (0),
                   (GtkAttachOptions) (GTK_FILL), 0, 0);

  popuser_label = gtk_label_new(_("User ::  "));
  gtk_widget_show(popuser_label);
  gtk_box_pack_start(GTK_BOX(popuser_hbox), popuser_label, FALSE, FALSE, 0);

  popuser_entry = gtk_entry_new();
  gtk_widget_show(popuser_entry);
  gtk_box_pack_start(GTK_BOX(popuser_hbox), popuser_entry, TRUE, TRUE, 0);

  poppass_hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(poppass_hbox);
  gtk_table_attach(GTK_TABLE(main_table), poppass_hbox, 0, 1, 4, 5,
                   (GtkAttachOptions) (0),
                   (GtkAttachOptions) (GTK_FILL), 0, 5);

  poppass_label = gtk_label_new(_("Pass ::  "));
  gtk_widget_show(poppass_label);
  gtk_box_pack_start(GTK_BOX(poppass_hbox), poppass_label, FALSE, FALSE, 0);

  poppass_entry = gtk_entry_new();
  gtk_widget_show(poppass_entry);
  gtk_box_pack_start(GTK_BOX(poppass_hbox), poppass_entry, TRUE, TRUE, 0);
  gtk_entry_set_visibility(GTK_ENTRY(poppass_entry), FALSE);

  popserver_hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(popserver_hbox);
  gtk_table_attach(GTK_TABLE(main_table), popserver_hbox, 0, 1, 5, 6,
                   (GtkAttachOptions) (0),
                   (GtkAttachOptions) (GTK_FILL), 0, 2);

  popserver_label = gtk_label_new(_("Pop server  ::       "));
  gtk_widget_show(popserver_label);
  gtk_box_pack_start(GTK_BOX(popserver_hbox), popserver_label, FALSE, FALSE,
                     0);
  gtk_misc_set_alignment(GTK_MISC(popserver_label), 0, 0.5);

  popserver_entry = gtk_entry_new();
  gtk_widget_show(popserver_entry);
  gtk_box_pack_start(GTK_BOX(popserver_hbox), popserver_entry, FALSE, TRUE,
                     0);
  gtk_widget_set_size_request(popserver_entry, 300, -1);

  popproto_hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(popproto_hbox);
  gtk_table_attach(GTK_TABLE(main_table), popproto_hbox, 0, 1, 7, 8,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 30, 2);

  popproto_label = gtk_label_new(_("whit protocol ::    "));
  gtk_widget_show(popproto_label);
  gtk_box_pack_start(GTK_BOX(popproto_hbox), popproto_label, FALSE, FALSE, 0);

  popproto_comboboxentry = gtk_combo_box_entry_new_text();
  gtk_widget_show(popproto_comboboxentry);
  gtk_box_pack_start(GTK_BOX(popproto_hbox), popproto_comboboxentry, FALSE,
                     TRUE, 0);
  gtk_combo_box_append_text(GTK_COMBO_BOX(popproto_comboboxentry),
                            _("AUTOPROBE"));

  gtk_combo_box_append_text(GTK_COMBO_BOX(popproto_comboboxentry),
                            _("POP3-SSL"));

  gtk_combo_box_append_text(GTK_COMBO_BOX(popproto_comboboxentry), _("POP3"));

  gtk_combo_box_append_text(GTK_COMBO_BOX(popproto_comboboxentry),
                            _("IMAP-SSL"));

  gtk_combo_box_append_text(GTK_COMBO_BOX(popproto_comboboxentry), _("IMAP"));

  buttons_hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(buttons_hbox);
  gtk_table_attach(GTK_TABLE(main_table), buttons_hbox, 0, 1, 9, 10,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 0);

  /*
   * apply_button = gtk_button_new_with_mnemonic (_("Apply"));
   * gtk_widget_show (apply_button); gtk_box_pack_start (GTK_BOX
   * (buttons_hbox), apply_button, TRUE, TRUE, 0);
   */

  cancel_button = gtk_button_new_with_mnemonic(_("Cancel"));
  gtk_widget_show(cancel_button);
  gtk_box_pack_start(GTK_BOX(buttons_hbox), cancel_button, TRUE, TRUE, 0);

  ok_button = gtk_button_new_with_mnemonic(_("Ok"));
  gtk_widget_show(ok_button);
  gtk_box_pack_start(GTK_BOX(buttons_hbox), ok_button, TRUE, TRUE, 0);

  keepon_checkbutton =
    gtk_check_button_new_with_mnemonic(_("Remove mail from server"));
  gtk_widget_show(keepon_checkbutton);
  gtk_table_attach(GTK_TABLE(main_table), keepon_checkbutton, 0, 1, 8, 9,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_EXPAND), 30, 0);

  /*
   * g_signal_connect ((gpointer) apply_button, "clicked", G_CALLBACK
   * (on_account_button_clicked), NULL);
   */

  g_signal_connect((gpointer) cancel_button, "clicked",
                   G_CALLBACK(on_accountcancel_button_clicked), NULL);
  g_signal_connect((gpointer) ok_button, "clicked",
                   G_CALLBACK(on_account_button_clicked), ok_button);

  return accounts_window;
}




GtkWidget *
create_backnext_window(struct backnext_pages * pages,
                       GtkWidget * finish_button)
{
  GtkWidget *backnext_window;
  GtkWidget *vbox;
  GtkWidget *scrolledwindow;
  GtkWidget *label;
  GtkWidget *hseparator;
  GtkWidget *out_hbox;
  GtkWidget *in_hbox;

  struct backnext_pages *p;
  GtkWidget *cancel_button;

  p = pages;

  backnext_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(backnext_window),
                       _("-= paranoy [backnext] =-"));
  gtk_widget_set_size_request(backnext_window, 450, 420);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox);
  gtk_container_add(GTK_CONTAINER(backnext_window), vbox);

  scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_show(scrolledwindow);
  gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, FALSE, TRUE, 0);
  gtk_widget_set_size_request(scrolledwindow, -1, 40);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                 GTK_POLICY_NEVER, GTK_POLICY_NEVER);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwindow),
                                      GTK_SHADOW_ETCHED_OUT);

  title_textview = gtk_text_view_new();
  gtk_widget_show(title_textview);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), title_textview);
  gtk_text_view_set_editable(GTK_TEXT_VIEW(title_textview), FALSE);
  gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(title_textview), FALSE);
  gtk_text_buffer_set_text(gtk_text_view_get_buffer
                           (GTK_TEXT_VIEW(title_textview)),
                           _("\n   Add account"), -1);

  gtk_widget_show_all(pages->page);

  while (p)
  {
    gtk_box_pack_start(GTK_BOX(vbox), p->page, TRUE, TRUE, 0);
    p = p->next;

  }

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_box_pack_start(GTK_BOX(vbox), hseparator, FALSE, TRUE, 5);

  out_hbox = gtk_hbox_new(TRUE, 0);
  gtk_widget_show(out_hbox);
  gtk_box_pack_start(GTK_BOX(vbox), out_hbox, FALSE, TRUE, 5);

  label = gtk_label_new(_(""));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(out_hbox), label, TRUE, FALSE, 0);


  in_hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(in_hbox);
  gtk_box_pack_start(GTK_BOX(out_hbox), in_hbox, TRUE, TRUE, 0);
  gtk_box_set_homogeneous((GtkBox *) in_hbox, TRUE);

  buio_label = gtk_label_new(_(""));
  gtk_widget_show(buio_label);
  gtk_box_pack_start(GTK_BOX(in_hbox), buio_label, TRUE, FALSE, 0);

  back_button = gtk_button_new_with_mnemonic(_("< Back "));
  // gtk_widget_show (back_button);
  gtk_box_pack_start(GTK_BOX(in_hbox), back_button, TRUE, FALSE, 0);
  gtk_widget_set_size_request(back_button, 65, -1);
  // gtk_button_set_relief(GTK_BUTTON(back_button), GTK_RELIEF_NONE);

  next_button = gtk_button_new_with_mnemonic(_(" Next >"));
  gtk_widget_show(next_button);
  gtk_box_pack_start(GTK_BOX(in_hbox), next_button, TRUE, FALSE, 0);
  gtk_widget_set_size_request(next_button, 65, -1);
  // gtk_button_set_relief(GTK_BUTTON(next_button), GTK_RELIEF_NONE);

  gtk_box_pack_start(GTK_BOX(in_hbox), finish_button, TRUE, FALSE, 0);
  gtk_widget_set_size_request(finish_button, 65, -1);

  cancel_button = gtk_button_new_with_mnemonic(_("Cancel"));
  gtk_widget_show(cancel_button);
  gtk_box_pack_start(GTK_BOX(in_hbox), cancel_button, TRUE, FALSE, 0);
  gtk_widget_set_size_request(cancel_button, 65, -1);
  // gtk_button_set_relief(GTK_BUTTON(cancel_button), GTK_RELIEF_NONE);

  g_signal_connect((gpointer) back_button, "clicked",
                   G_CALLBACK(on_back_button_clicked), (gpointer) pages);

  g_signal_connect((gpointer) next_button, "clicked",
                   G_CALLBACK(on_next_button_clicked), (gpointer) pages);
  g_signal_connect((gpointer) cancel_button, "clicked",
                   G_CALLBACK(on_newcancel_button_clicked), NULL);

  return backnext_window;
}

struct backnext_pages *
add_account_pages_new()
{
  struct backnext_pages *pages, *p;
  GtkWidget *table, *label, *hbox, *hseparator;
  GSList *keep_radiobutton_group = NULL;
  GSList *pop_radiobutton_group = NULL;
  GSList *clear_radiobutton_group = NULL;

  // Page 1

  pages = (struct backnext_pages *) malloc(sizeof(struct backnext_pages));
  p = pages;

  pages->prev = NULL;

  pages->page = gtk_alignment_new(0.5, 0.5, 1, 1);
  gtk_alignment_set_padding(GTK_ALIGNMENT(pages->page), 20, 15, 30, 30);

  table = gtk_table_new(4, 1, FALSE);
  gtk_container_add(GTK_CONTAINER(pages->page), table);

  label =
    gtk_label_new(_
                  ("Enter your name, the one which will be used in the <b>From:</b> field\n[as \"antani\"]"));
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0,
                   0);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);

  label =
    gtk_label_new(_
                  ("Enter your email address, the one which is used by others to        \nsend mails to you\n[as \"antani@tarapiatapioco.it\"]"));
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0,
                   0);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 3, 4,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 35);

  addacc_email_entry = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox), addacc_email_entry, TRUE, TRUE, 0);

  label = gtk_label_new(_(" :: Your email"));
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 35);

  addacc_name_entry = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox), addacc_name_entry, TRUE, TRUE, 0);

  label = gtk_label_new(_(" :: Your name"));
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  // Page 2

  pages->next =
    (struct backnext_pages *) malloc(sizeof(struct backnext_pages));
  pages->next->prev = pages;

  pages = pages->next;

  pages->page = gtk_alignment_new(0.5, 0.5, 1, 1);
  gtk_alignment_set_padding(GTK_ALIGNMENT(pages->page), 20, 15, 30, 30);

  table = gtk_table_new(6, 1, FALSE);
  gtk_container_add(GTK_CONTAINER(pages->page), table);

  label =
    gtk_label_new(_
                  ("Enter address and port of your incoming server, where paranoy will\ncheck for new mails.\n[as \"pop.tarapiatapioco.it\"\n       \"110\" is the default port for clear connections\n       \"995\" is the default port for SSL encrypted connections]\t            "));
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0,
                   0);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 30);

  label = gtk_label_new(_("Server :: "));
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  addacc_pop_entry = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox), addacc_pop_entry, TRUE, TRUE, 0);

  label = gtk_label_new(_(" :: "));
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  addacc_port_entry = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox), addacc_port_entry, FALSE, TRUE, 0);
  gtk_widget_set_size_request(addacc_port_entry, 50, -1);

  label = gtk_label_new(_(" :: Port"));
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  label = gtk_label_new(_("Select the type of incoming server"));
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 3, 4,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 5);

  pop_radiobutton = gtk_radio_button_new_with_mnemonic(NULL, _("POP   "));
  gtk_box_pack_start(GTK_BOX(hbox), pop_radiobutton, FALSE, FALSE, 0);
  gtk_radio_button_set_group(GTK_RADIO_BUTTON(pop_radiobutton),
                             pop_radiobutton_group);
  pop_radiobutton_group =
    gtk_radio_button_get_group(GTK_RADIO_BUTTON(pop_radiobutton));

  imap_radiobutton = gtk_radio_button_new_with_mnemonic(NULL, _("IMAP"));
  gtk_box_pack_start(GTK_BOX(hbox), imap_radiobutton, FALSE, FALSE, 0);
  gtk_radio_button_set_group(GTK_RADIO_BUTTON(imap_radiobutton),
                             pop_radiobutton_group);
  pop_radiobutton_group =
    gtk_radio_button_get_group(GTK_RADIO_BUTTON(imap_radiobutton));

  label = gtk_label_new(_("Select the type of connection"));
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 5, 6,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 5);

  clear_radiobutton = gtk_radio_button_new_with_mnemonic(NULL, _("Clear   "));
  gtk_box_pack_start(GTK_BOX(hbox), clear_radiobutton, FALSE, FALSE, 0);
  gtk_radio_button_set_group(GTK_RADIO_BUTTON(clear_radiobutton),
                             clear_radiobutton_group);
  clear_radiobutton_group =
    gtk_radio_button_get_group(GTK_RADIO_BUTTON(clear_radiobutton));

  ssl_radiobutton =
    gtk_radio_button_new_with_mnemonic(NULL, _("SSL Encrypted"));
  gtk_box_pack_start(GTK_BOX(hbox), ssl_radiobutton, FALSE, FALSE, 0);
  gtk_radio_button_set_group(GTK_RADIO_BUTTON(ssl_radiobutton),
                             clear_radiobutton_group);
  clear_radiobutton_group =
    gtk_radio_button_get_group(GTK_RADIO_BUTTON(ssl_radiobutton));

  autoprobe_button = gtk_button_new_with_mnemonic(_(" Autoprobe "));
  gtk_box_pack_end(GTK_BOX(hbox), autoprobe_button, FALSE, FALSE, 2);

  // Page 3

  pages->next =
    (struct backnext_pages *) malloc(sizeof(struct backnext_pages));
  pages->next->prev = pages;

  pages = pages->next;

  pages->page = gtk_alignment_new(0.5, 0.5, 1, 1);
  gtk_alignment_set_padding(GTK_ALIGNMENT(pages->page), 20, 15, 30, 30);

  table = gtk_table_new(9, 1, FALSE);
  gtk_container_add(GTK_CONTAINER(pages->page), table);

  label =
    gtk_label_new(_
                  ("Enter the username which paranoy will use to connect to your\nincoming server\n[as \"g.antani\"]"));
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0,
                   0);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  label =
    gtk_label_new(_
                  ("Enter the password which paranoy will use for authentication on your\nincoming server\n[leave white this field to not save it]"));
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0,
                   0);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 20);

  username_entry = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox), username_entry, TRUE, TRUE, 0);
  gtk_widget_set_size_request(username_entry, 240, -1);

  label = gtk_label_new(_(" :: Username"));
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 3, 4,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 20);

  password_entry = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox), password_entry, TRUE, TRUE, 0);
  gtk_widget_set_size_request(password_entry, 240, -1);
  gtk_entry_set_visibility(GTK_ENTRY(password_entry), FALSE);

  label = gtk_label_new(_(" :: Password"));
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  hseparator = gtk_hseparator_new();
  gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 4, 5,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 3);

  keep_radiobutton =
    gtk_radio_button_new_with_mnemonic(NULL, _("Keep mails on server"));
  gtk_table_attach(GTK_TABLE(table), keep_radiobutton, 0, 1, 5, 6,
                   (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0,
                   0);
  gtk_radio_button_set_group(GTK_RADIO_BUTTON(keep_radiobutton),
                             keep_radiobutton_group);
  keep_radiobutton_group =
    gtk_radio_button_get_group(GTK_RADIO_BUTTON(keep_radiobutton));

  delete_radiobutton =
    gtk_radio_button_new_with_mnemonic(NULL,
                                       _("Delete mails from server ::"));
  gtk_table_attach(GTK_TABLE(table), delete_radiobutton, 0, 1, 6, 7,
                   (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0,
                   0);
  gtk_radio_button_set_group(GTK_RADIO_BUTTON(delete_radiobutton),
                             keep_radiobutton_group);
  // keep_radiobutton_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON
  // (delete_radiobutton));

  deldownload_radiobutton =
    gtk_radio_button_new_with_mnemonic(NULL, _("when paranoy download them"));
  gtk_table_attach(GTK_TABLE(table), deldownload_radiobutton, 0, 1, 7, 8,
                   (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 20,
                   0);
  gtk_radio_button_set_group(GTK_RADIO_BUTTON(deldownload_radiobutton),
                             keep_radiobutton_group);
  // keep_radiobutton_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON
  // (deldownload_radiobutton));

  delI_radiobutton =
    gtk_radio_button_new_with_mnemonic(NULL,
                                       _("when I delete them from inbox"));
  gtk_table_attach(GTK_TABLE(table), delI_radiobutton, 0, 1, 8, 9,
                   (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 20,
                   0);
  gtk_radio_button_set_group(GTK_RADIO_BUTTON(delI_radiobutton),
                             keep_radiobutton_group);
  // keep_radiobutton_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON
  // (delI_radiobutton));

  // Page 4

  pages->next =
    (struct backnext_pages *) malloc(sizeof(struct backnext_pages));
  pages->next->prev = pages;
  pages = pages->next;

  pages->page = gtk_alignment_new(0.5, 0.5, 1, 1);
  gtk_alignment_set_padding(GTK_ALIGNMENT(pages->page), 20, 15, 30, 30);

  confirm_table = gtk_table_new(3, 1, FALSE);
  gtk_container_add(GTK_CONTAINER(pages->page), confirm_table);

  label = gtk_label_new(_("Check if the information below is correct"));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(confirm_table), label, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);


  label = gtk_label_new(_("Click Finsh to save your new account."));
  gtk_table_attach(GTK_TABLE(confirm_table), label, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 0, 40);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  pages->next = NULL;
  return p;

}

GtkWidget *
create_twobutton_dialog(char *message, void *ok_handler)
{

  GtkWidget *dialog_vbox, *dialog;
  GtkWidget *hbox;
  GtkWidget *image;
  GtkWidget *label;
  GtkWidget *dialog_action_area;
  GtkWidget *cancelbutton;
  GtkWidget *okbutton;

  dialog = gtk_dialog_new();
  // gtk_widget_set_size_request (account_dialog, 530, 120);
  gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_title(GTK_WINDOW(dialog), _("-= ??? =-"));
  gtk_window_set_type_hint(GTK_WINDOW(dialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox = GTK_DIALOG(dialog)->vbox;
  gtk_widget_show(dialog_vbox);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(dialog_vbox), hbox, TRUE, TRUE, 15);

  image = gtk_image_new_from_stock("gtk-dialog-warning", GTK_ICON_SIZE_DND);
  gtk_widget_show(image);
  gtk_box_pack_start(GTK_BOX(hbox), image, TRUE, TRUE, 20);

  label = gtk_label_new(_(message));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 20);

  dialog_action_area = GTK_DIALOG(dialog)->action_area;
  gtk_widget_show(dialog_action_area);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area),
                            GTK_BUTTONBOX_END);

  cancelbutton = gtk_button_new_with_label("Cancel");
  gtk_widget_show(cancelbutton);
  gtk_dialog_add_action_widget(GTK_DIALOG(dialog), cancelbutton,
                               GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS(cancelbutton, GTK_CAN_DEFAULT);

  okbutton = gtk_button_new_with_label("Ok");
  gtk_widget_show(okbutton);
  gtk_dialog_add_action_widget(GTK_DIALOG(dialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS(okbutton, GTK_CAN_DEFAULT);

  g_signal_connect((gpointer) cancelbutton, "clicked",
                   G_CALLBACK(on_cancel_button_clicked), (gpointer) dialog);
  g_signal_connect((gpointer) okbutton, "clicked",
                   G_CALLBACK(ok_handler), (gpointer) dialog);


  return dialog;
}



GtkWidget *
create_contact_window(int mode)
{
  GtkWidget *contact_window;
  GtkWidget *notebook;
  GtkWidget *table;
  GtkWidget *label;
  GtkWidget *hbox;
  GtkWidget *hseparator;
  GtkWidget *received_tree, *sent_tree, *sent_sel, *received_sel,
    *received_store, *sent_store;
  GtkWidget *scrolledwindow;
  GtkWidget *descr_textview;
  GtkWidget *last_hbox;
  GtkWidget *viewport;
  GtkTreeIter *iter;
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *col;

  GtkWidget *field_combobox;
  int *where;

  where = (int *) malloc(sizeof(int));
  *(where) = 0;

  contact_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(contact_window), _("-= Contact =-"));
  gtk_window_set_position(GTK_WINDOW(contact_window),
                          GTK_WIN_POS_CENTER_ON_PARENT);

  notebook = gtk_notebook_new();
  gtk_widget_show(notebook);
  gtk_container_add(GTK_CONTAINER(contact_window), notebook);

  table = gtk_table_new(9, 1, FALSE);
  gtk_widget_show(table);
  gtk_container_add(GTK_CONTAINER(notebook), table);

  label =
    gtk_label_new(_
                  ("<b>:: Display name ::</b>\nThis is the name used by paranoy to refer this contact"));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                   (GtkAttachOptions) (0), 20, 15);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 1, 2,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 20, 0);

  contactname_entry = gtk_entry_new();
  gtk_widget_show(contactname_entry);
  gtk_box_pack_start(GTK_BOX(hbox), contactname_entry, TRUE, TRUE, 0);

  label = gtk_label_new(_(" :: Name"));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  label =
    gtk_label_new(_("<b>:: Email address ::</b>\nThis is contact's email"));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 20, 15);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 3, 4,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 20, 0);

  contactmail_entry = gtk_entry_new();
  gtk_widget_show(contactmail_entry);
  gtk_box_pack_start(GTK_BOX(hbox), contactmail_entry, TRUE, TRUE, 0);

  label = gtk_label_new(_(" :: Email"));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 4, 5,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 25, 10);

  label =
    gtk_label_new(_
                  ("Here you can add additional information.\nParanoy's staff is very sensible on privacy stuff so would be\nbetter if you will be carefully writing here [TOFIX]"));
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (0), 20, 5);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_show(scrolledwindow);
  gtk_table_attach(GTK_TABLE(table), scrolledwindow, 0, 1, 6, 7,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 20, 5);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwindow),
                                      GTK_SHADOW_IN);

  descr_textview = gtk_text_view_new();
  gtk_widget_show(descr_textview);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), descr_textview);

  hseparator = gtk_hseparator_new();
  gtk_widget_show(hseparator);
  gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 7, 8,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);

  last_hbox =
    create_common_button_box(on_contactok_button_clicked, contact_window);
  gtk_widget_show_all(last_hbox);
  gtk_table_attach(GTK_TABLE(table), last_hbox, 0, 1, 8, 9,
                   (GtkAttachOptions) (GTK_FILL),
                   (GtkAttachOptions) (GTK_FILL), 0, 10);

  label = gtk_label_new(_("<b>Infoz</b>"));
  gtk_widget_show(label);
  gtk_notebook_set_tab_label(GTK_NOTEBOOK(notebook),
                             gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),
                                                       0), label);
  gtk_widget_set_size_request(label, 40, -1);
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);

  if (1 == mode)
  {

    table = gtk_table_new(9, 1, FALSE);
    gtk_widget_show(table);
    gtk_container_add(GTK_CONTAINER(notebook), table);


    label = gtk_label_new(_("<b>:: Received ::</b>"));
    gtk_widget_show(label);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (0), 20, 15);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

    scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
    gtk_widget_show(scrolledwindow);
    gtk_table_attach(GTK_TABLE(table), scrolledwindow, 0, 1, 1, 2,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 20, 0);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);

    viewport = gtk_viewport_new(NULL, NULL);
    gtk_widget_show(viewport);
    gtk_container_add(GTK_CONTAINER(scrolledwindow), viewport);

    // received tree
    received_store = (GtkWidget *)
      gtk_list_store_new(MAIL_NCOL, GDK_TYPE_PIXBUF, G_TYPE_STRING,
                         G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER,
                         G_TYPE_BOOLEAN);

    if (current_fold)
      print_mails(received_store, current_fold, 13, current_contact->email);

    received_tree = gtk_tree_view_new();
    gtk_tree_view_set_model(GTK_TREE_VIEW(received_tree),
                            GTK_TREE_MODEL(received_store));
    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(received_tree), FALSE);


    gtk_widget_show(received_tree);
    gtk_container_add(GTK_CONTAINER(viewport), received_tree);


    renderer = gtk_cell_renderer_pixbuf_new();

    col =
      gtk_tree_view_column_new_with_attributes("", renderer, "pixbuf",
                                               MAIL_ATTACH, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(received_tree), col);
    gtk_tree_view_column_set_fixed_width(GTK_TREE_VIEW_COLUMN(col), 2);

    renderer = gtk_cell_renderer_text_new();
    g_object_set(renderer, "size", 8000, "size-set", TRUE, NULL);

    col =
      gtk_tree_view_column_new_with_attributes("Date", renderer, "text",
                                               MAIL_DATE, NULL);
    gtk_tree_view_column_set_cell_data_func(col, renderer, mail_renderer,
                                            NULL, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(received_tree), col);
    gtk_tree_view_column_set_min_width(GTK_TREE_VIEW_COLUMN(col), 50);
    // gtk_tree_view_column_set_max_width(GTK_TREE_VIEW_COLUMN(col), 35);
    // gtk_tree_view_column_set_expand(GTK_TREE_VIEW_COLUMN(col), TRUE);
    gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(col), TRUE);

    col =
      gtk_tree_view_column_new_with_attributes("Subject", renderer, "text",
                                               MAIL_SUBJECT, NULL);
    gtk_tree_view_column_set_cell_data_func(col, renderer, mail_renderer,
                                            NULL, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(received_tree), col);
    // gtk_tree_view_column_set_min_width(GTK_TREE_VIEW_COLUMN(col), 295);
    gtk_tree_view_column_set_expand(GTK_TREE_VIEW_COLUMN(col), TRUE);
    gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(col), TRUE);
    gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(col),
                                    GTK_TREE_VIEW_COLUMN_FIXED);

    // print mails

    // gtk_tree_model_get_iter_first(GTK_TREE_MODEL(received_store), &iter);
    // received_sel =
    // gtk_tree_view_get_selection(GTK_TREE_VIEW(received_tree));
    // gtk_tree_selection_select_iter(GTK_TREE_SELECTION(received_sel),
    // &iter);

    label = gtk_label_new(_("<b>:: Sent ::</b>"));
    gtk_widget_show(label);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 20, 15);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

    scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
    gtk_widget_show(scrolledwindow);
    gtk_table_attach(GTK_TABLE(table), scrolledwindow, 0, 1, 3, 4,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 20, 0);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);

    viewport = gtk_viewport_new(NULL, NULL);
    gtk_widget_show(viewport);
    gtk_container_add(GTK_CONTAINER(scrolledwindow), viewport);

    // sent tree

    sent_store = (GtkWidget *)
      gtk_list_store_new(MAIL_NCOL, GDK_TYPE_PIXBUF, G_TYPE_STRING,
                         G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER,
                         G_TYPE_BOOLEAN);

    if (current_fold)
      print_mails(sent_store, current_fold, 14, current_contact->email);

    sent_tree = gtk_tree_view_new();
    gtk_tree_view_set_model(GTK_TREE_VIEW(sent_tree),
                            GTK_TREE_MODEL(sent_store));
    gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(sent_tree), TRUE);
    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(sent_tree), FALSE);


    gtk_widget_show(sent_tree);
    gtk_container_add(GTK_CONTAINER(viewport), sent_tree);


    renderer = gtk_cell_renderer_pixbuf_new();

    col =
      gtk_tree_view_column_new_with_attributes("", renderer, "pixbuf",
                                               MAIL_ATTACH, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(sent_tree), col);
    gtk_tree_view_column_set_fixed_width(GTK_TREE_VIEW_COLUMN(col), 2);

    renderer = gtk_cell_renderer_text_new();
    g_object_set(renderer, "size", 8000, "size-set", TRUE, NULL);

    col =
      gtk_tree_view_column_new_with_attributes("Date", renderer, "text",
                                               MAIL_DATE, NULL);
    gtk_tree_view_column_set_cell_data_func(col, renderer, mail_renderer,
                                            NULL, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(sent_tree), col);
    gtk_tree_view_column_set_min_width(GTK_TREE_VIEW_COLUMN(col), 50);
    // gtk_tree_view_column_set_max_width(GTK_TREE_VIEW_COLUMN(col), 35);
    // gtk_tree_view_column_set_expand(GTK_TREE_VIEW_COLUMN(col), TRUE);
    gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(col), TRUE);

    col =
      gtk_tree_view_column_new_with_attributes("Subject", renderer, "text",
                                               MAIL_SUBJECT, NULL);
    gtk_tree_view_column_set_cell_data_func(col, renderer, mail_renderer,
                                            NULL, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(sent_tree), col);
    // gtk_tree_view_column_set_min_width(GTK_TREE_VIEW_COLUMN(col), 295);
    gtk_tree_view_column_set_expand(GTK_TREE_VIEW_COLUMN(col), TRUE);
    gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(col), TRUE);
    gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(col),
                                    GTK_TREE_VIEW_COLUMN_FIXED);

    // print mails

    // gtk_tree_model_get_iter_first(GTK_TREE_MODEL(received_store), &iter);
    // received_sel =
    // gtk_tree_view_get_selection(GTK_TREE_VIEW(received_tree));
    // gtk_tree_selection_select_iter(GTK_TREE_SELECTION(received_sel),
    // &iter);

    hseparator = gtk_hseparator_new();
    gtk_widget_show(hseparator);
    gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 4, 5,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 25, 10);

    label =
      gtk_label_new(_
                    ("<b>:: Search ::</b>\nType here to search in those groups of mails"));
    gtk_widget_show(label);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 20, 0);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

    hbox = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(hbox);
    gtk_table_attach(GTK_TABLE(table), hbox, 0, 1, 6, 7,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 20, 10);

    label = gtk_label_new(_("Look for :: "));
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

    search_entry = gtk_entry_new();
    gtk_widget_show(search_entry);
    gtk_box_pack_start(GTK_BOX(hbox), search_entry, TRUE, TRUE, 0);

    field_combobox = gtk_combo_box_new_text();
    gtk_widget_show(field_combobox);
    gtk_box_pack_start(GTK_BOX(hbox), field_combobox, TRUE, TRUE, 0);
    gtk_combo_box_append_text(GTK_COMBO_BOX(field_combobox), _("From"));
    gtk_combo_box_append_text(GTK_COMBO_BOX(field_combobox), _("Subject"));
    gtk_combo_box_append_text(GTK_COMBO_BOX(field_combobox), _("Both"));
    gtk_combo_box_set_active(GTK_COMBO_BOX(field_combobox), 0);

    hseparator = gtk_hseparator_new();
    gtk_widget_show(hseparator);
    gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 7, 8,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 10);

    last_hbox =
      create_common_button_box(on_contactok_button_clicked, contact_window);

    gtk_widget_show_all(last_hbox);
    gtk_table_attach(GTK_TABLE(table), last_hbox, 0, 1, 8, 9,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 10);

    label = gtk_label_new(_("<b>Mails</b>"));
    gtk_widget_show(label);
    gtk_notebook_set_tab_label(GTK_NOTEBOOK(notebook),
                               gtk_notebook_get_nth_page(GTK_NOTEBOOK
                                                         (notebook), 1),
                               label);
    gtk_widget_set_size_request(label, 40, -1);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);

    table = gtk_table_new(3, 1, FALSE);
    gtk_widget_show(table);
    gtk_container_add(GTK_CONTAINER(notebook), table);

    label =
      gtk_label_new(_
                    ("Qui stampiamo la chiave pubblica se c'e', ciccia se nn c'e'"));
    gtk_widget_show(label);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND), 20, 0);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

    hseparator = gtk_hseparator_new();
    gtk_widget_show(hseparator);
    gtk_table_attach(GTK_TABLE(table), hseparator, 0, 1, 1, 2,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 10);

    last_hbox =
      create_common_button_box(on_contactok_button_clicked, contact_window);
    gtk_widget_show_all(last_hbox);
    gtk_table_attach(GTK_TABLE(table), last_hbox, 0, 1, 2, 3,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 10);

    label = gtk_label_new(_("<b>PGP</b>"));
    gtk_widget_show(label);
    gtk_notebook_set_tab_label(GTK_NOTEBOOK(notebook),
                               gtk_notebook_get_nth_page(GTK_NOTEBOOK
                                                         (notebook), 2),
                               label);
    gtk_widget_set_size_request(label, 40, -1);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);


    gtk_entry_set_text((GtkEntry *) contactname_entry, current_contact->name);
    gtk_entry_set_text((GtkEntry *) contactmail_entry,
                       current_contact->email);

    g_signal_connect((gpointer) received_tree, "row_activated",
                     G_CALLBACK(on_open_mail_activate), received_store);
    printf("where is %d\n", *where);
    *where++;
    g_signal_connect((gpointer) sent_tree, "row_activated",
                     G_CALLBACK(on_open_mail_activate), sent_store);
  }

  g_signal_connect((gpointer) contact_window, "destroy_event",
                   G_CALLBACK(on_cancel_button_clicked), contact_window);

  return contact_window;
}


GtkWidget*
create_addfilter_window (void)
{
  GtkWidget *addfilter_window;
  GtkWidget *table;
  GtkWidget *hseparator;
  GtkWidget *label;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *equal_combobox;
  GtkWidget *what_entry;
  GtkWidget *where_combobox;
  GtkWidget *addrole_button;
  GtkWidget *update_button;
  GtkWidget *delrole_button;
  GtkWidget *radiobutton;
  GSList *radiobutton_group = NULL;
  GtkWidget *scrolledwindow;
  GtkWidget *viewport;
  GtkWidget *action_table;
  GtkWidget *copy_checkbutton;
  GtkWidget *mark_checkbutton;
  GtkWidget *forward_checkbutton;
  GtkWidget *move_checkbutton;
  GtkWidget *move_combobox;
  GtkWidget *copy_combobox;
  GtkWidget *forward_entry;
  GtkWidget *delete_checkbutton;
  GtkWidget *mark_combobox;
  GtkWidget *info_label;
  GtkWidget *filname_entry;
  
  GtkWidget *filter_tree;
  GtkTreeViewColumn *col;
  GtkCellRenderer *renderer;

  addfilter_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_size_request (addfilter_window, 470, 571);
  gtk_window_set_title (GTK_WINDOW (addfilter_window), _(":: paranoy ::"));
  gtk_window_set_position (GTK_WINDOW (addfilter_window), GTK_WIN_POS_CENTER_ON_PARENT);

  table = gtk_table_new (12, 1, FALSE);
  gtk_widget_show (table);
  gtk_container_add (GTK_CONTAINER (addfilter_window), table);

  hseparator = gtk_hseparator_new ();
  gtk_widget_show (hseparator);
  gtk_table_attach (GTK_TABLE (table), hseparator, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 140, 0);

  label = gtk_label_new (_(":<b>:: what mails ::</b>"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 30, 10);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gtk_table_attach (GTK_TABLE (table), vbox, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 30, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);

  equal_combobox = gtk_combo_box_new_text ();
  gtk_widget_show (equal_combobox);
  gtk_widget_set_size_request (equal_combobox,110,-1);
  gtk_box_pack_start (GTK_BOX (hbox), equal_combobox, FALSE, TRUE, 0);
  gtk_combo_box_append_text (GTK_COMBO_BOX (equal_combobox), _("contain"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (equal_combobox), _("not contain"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (equal_combobox), _("equal to"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (equal_combobox), _("not equal to"));
  gtk_combo_box_set_active (GTK_COMBO_BOX (equal_combobox), 0);

  what_entry = gtk_entry_new ();
  gtk_widget_show (what_entry);
  gtk_box_pack_start (GTK_BOX (hbox), what_entry, TRUE, TRUE, 0);

  where_combobox = gtk_combo_box_new_text ();
  gtk_widget_show (where_combobox);
  gtk_widget_set_size_request (where_combobox,85,-1);
  gtk_box_pack_start (GTK_BOX (hbox), where_combobox, FALSE, TRUE, 0);
  gtk_combo_box_append_text (GTK_COMBO_BOX (where_combobox), _("Subject"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (where_combobox), _("From"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (where_combobox), _("To"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (where_combobox), _("Reply-to"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (where_combobox), _("CC"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (where_combobox), _("Bcc"));
  gtk_combo_box_set_active (GTK_COMBO_BOX (where_combobox), 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);

  addrole_button = gtk_button_new_with_mnemonic (_("Add rule"));
  gtk_widget_show (addrole_button);
  gtk_box_pack_start (GTK_BOX (hbox), addrole_button, TRUE, TRUE, 0);

  update_button = gtk_button_new_with_mnemonic (_("Update rule"));
  gtk_widget_show (update_button);
  gtk_box_pack_start (GTK_BOX (hbox), update_button, TRUE, TRUE, 0);

  delrole_button = gtk_button_new_with_mnemonic (_("Delete rule"));
  gtk_widget_show (delrole_button);
  gtk_box_pack_start (GTK_BOX (hbox), delrole_button, TRUE, TRUE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 30, 5);

  radiobutton = gtk_radio_button_new_with_mnemonic (NULL, _("All of these rules"));
  gtk_widget_show (radiobutton);
  gtk_box_pack_start (GTK_BOX (hbox), radiobutton, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton), radiobutton_group);
  radiobutton_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton));

  radiobutton = gtk_radio_button_new_with_mnemonic (NULL, _("Any of these rules"));
  gtk_widget_show (radiobutton);
  gtk_box_pack_start (GTK_BOX (hbox), radiobutton, TRUE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton), radiobutton_group);
  radiobutton_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton));

  hseparator = gtk_hseparator_new ();
  gtk_widget_show (hseparator);
  gtk_table_attach (GTK_TABLE (table), hseparator, 0, 1, 7, 8,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 140, 10);

  hseparator = gtk_hseparator_new ();
  gtk_widget_show (hseparator);
  gtk_table_attach (GTK_TABLE (table), hseparator, 0, 1, 10, 11,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 10);

  hbox =   create_common_button_box(on_addfilterok_button_clicked, addfilter_window);
  gtk_widget_show_all(hbox);
  gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 11, 12,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 10);

  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow);
  gtk_table_attach (GTK_TABLE (table), scrolledwindow, 0, 1, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 30, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  viewport = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport);
  gtk_container_add (GTK_CONTAINER (scrolledwindow), viewport);
  
  filter_tree = gtk_tree_view_new();
  gtk_widget_show(filter_tree);
  gtk_container_add(GTK_CONTAINER(viewport), filter_tree);
  gtk_tree_view_set_headers_visible((GtkTreeView *)filter_tree,TRUE);
  
  renderer = gtk_cell_renderer_text_new();
  
  col = gtk_tree_view_column_new_with_attributes("Must", renderer, "text",
                                             FILT_MUST, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(filter_tree), col);
//  gtk_tree_view_column_set_fixed_width(GTK_TREE_VIEW_COLUMN(mail_col), 17);
  
col = gtk_tree_view_column_new_with_attributes("What", renderer, "text",
                                             FILT_WHAT, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(filter_tree), col);

  col = gtk_tree_view_column_new_with_attributes("Where", renderer, "text",
                                             FILT_WHERE, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(filter_tree), col);

  label = gtk_label_new (_("<b>:: what i have to do ::</b>"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 8, 9,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 30, 5);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 9, 10,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 30, 0);
  gtk_widget_set_size_request (hbox, -1, 140);

  action_table = gtk_table_new (5, 2, FALSE);
  gtk_widget_show (action_table);
  gtk_box_pack_start (GTK_BOX (hbox), action_table, FALSE, FALSE, 0);

  copy_checkbutton = gtk_check_button_new_with_mnemonic (_("Copy in ::"));
  gtk_widget_show (copy_checkbutton);
  gtk_table_attach (GTK_TABLE (action_table), copy_checkbutton, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  mark_checkbutton = gtk_check_button_new_with_mnemonic (_("Mark as ::"));
  gtk_widget_show (mark_checkbutton);
  gtk_table_attach (GTK_TABLE (action_table), mark_checkbutton, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  forward_checkbutton = gtk_check_button_new_with_mnemonic (_("Forward to ::"));
  gtk_widget_show (forward_checkbutton);
  gtk_table_attach (GTK_TABLE (action_table), forward_checkbutton, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  move_checkbutton = gtk_check_button_new_with_mnemonic (_("Move in ::"));
  gtk_widget_show (move_checkbutton);
  gtk_table_attach (GTK_TABLE (action_table), move_checkbutton, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_size_request (move_checkbutton, 100, -1);

  move_combobox = gtk_combo_box_new ();
  gtk_widget_show (move_combobox);
  gtk_table_attach (GTK_TABLE (action_table), move_combobox, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (move_combobox, 140, -1);

  copy_combobox = gtk_combo_box_new ();
  gtk_widget_show (copy_combobox);
  gtk_table_attach (GTK_TABLE (action_table), copy_combobox, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (copy_combobox, 140, -1);

  forward_entry = gtk_entry_new ();
  gtk_widget_show (forward_entry);
  gtk_table_attach (GTK_TABLE (action_table), forward_entry, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  delete_checkbutton = gtk_check_button_new_with_mnemonic (_("Delete"));
  gtk_widget_show (delete_checkbutton);
  gtk_table_attach (GTK_TABLE (action_table), delete_checkbutton, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  mark_combobox = gtk_combo_box_new_text ();
  gtk_widget_show (mark_combobox);
  gtk_table_attach (GTK_TABLE (action_table), mark_combobox, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_combo_box_append_text (GTK_COMBO_BOX (mark_combobox), _("New"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (mark_combobox), _("Readed"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (mark_combobox), _("Important"));
  gtk_combo_box_set_active (GTK_COMBO_BOX (mark_combobox), 0);

  info_label = gtk_label_new (_("Qui\nstampa info"));
  gtk_widget_show (info_label);
  gtk_box_pack_start (GTK_BOX (hbox), info_label, TRUE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (info_label), GTK_JUSTIFY_CENTER);

  label = gtk_label_new (_("<b>:: Add filter ::</b>"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 30, 15);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 30, 10);
  
  
  
  filname_entry = gtk_entry_new ();
  gtk_widget_show (filname_entry);
  gtk_box_pack_start (GTK_BOX (hbox), filname_entry, TRUE, TRUE, 0);

  label = gtk_label_new (_("  :: Filter name"));
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

  g_signal_connect ((gpointer) addrole_button, "clicked",
                    G_CALLBACK (on_addrole_button_clicked),
                    NULL);
  g_signal_connect ((gpointer) delrole_button, "clicked",
                    G_CALLBACK (on_delrole_button_clicked),
                    NULL);

  return addfilter_window;
}

