  /*
   * Copyright (C) 2003  
   * paranoy project - paranox.sf.net - paranoy@autistici.org
   *
   *    This program is free software; you can redistribute it and/or modify
   *    it under the terms of the GNU General Public License as published by
   *    the Free Software Foundation; either version 2 of the License, or
   *    (at your option) any later version.
   *
   *    This program is distributed in the hope that it will be useful,
   *    but WITHOUT ANY WARRANTY; without even the implied warranty of
   *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   *    GNU General Public License for more details.
   *
   *    You should have received a copy of the GNU General Public License
   *    along with this program; if not, write to the Free Software
   *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
   */



#include  "include.h"

int
main(int narg, char *arg[])
{
  byte i;
  char *s;
  char buff[BUFFER_REAL];
  FILE *conf;

#ifdef USE_GTK
  THEME = NULL;
#endif

  ERR_load_SSL_strings();
  ERR_load_BIO_strings();
  ERR_load_crypto_strings();

  /*
   * SSL_library_init() registers the available ciphers and digests.
   * OpenSSL_add_ssl_algorithms() and SSLeay_add_ssl_algorithms() are
   * synonyms for SSL_library_init(). SSL_library_init() must be called
   * before any other action takes place. SSL_library_init() only
   * registers ciphers. Another important initialization is the seeding
   * of the PRNG (Pseudo Random Number Generator), which has to be
   * performed separately. SSL_library_init() always returns "1", so it
   * is safe to discard the return value. 
   */
  SSLeay_add_ssl_algorithms();

  // choose default gui
#ifdef  USE_GTK
  gui = GTK;
#else
  gui = NCURSES;
  stdscr = NULL;
#endif

  // parse command line arguments
  for (i = 1; i < narg; i++)
  {
    s = arg[i];
    if (s[0] == '-' && s[1] == '-')
    {
      s += 2;
      if (!strcmp(s, "version"))
      {
        fprintf(stderr, "\n\e[31;1m  paranoy %s\e[0;m\n\n", VERSION);
        return 0;
      } else if (!strcmp(s, "compose"))
      {
        mode = COMPOSE;
        break;
      }
#ifdef USE_GTK
      else if (!strcmp(s, "theme"))
      {
        if (narg < i)
        {
          fprintf(stderr, "\n\t--theme options need an argoument\n\n");
          return 0;
        }
        THEME = strdup(arg[++i]);
      }
#endif
      else if (!strcmp(s, "gui"))
      {
        if (narg < i)
        {
          fprintf(stderr, "\n\t--gui options need an argoument\n\n");
          return 0;
        }
        if (!strcmp(arg[i + 1], "curses"))
        {
          gui = NCURSES;
          i++;
        } else if (!strcmp(arg[i + 1], "gtk"))
        {
          gui = GTK;
          i++;
        } else
        {
          usage();
          return 0;
        }
        break;
      } else
      {
        usage();
        return 0;
      }
    } else if (s[0] == '-')
    {
      switch (s[1])
      {
        case 'v':
          fprintf(stderr, "\n\e[31;1m  paranoy %s\e[0;m\n\n", VERSION);
          return 0;
        case 'c':
          mode = COMPOSE;
          break;
        case 'h':
          usage();
          return 0;
        case 'g':
          if (narg < i)
          {
            fprintf(stderr, "\n\t-g options need an argoument\n\n");
            return 0;

          }
          if (!strcmp(arg[i + 1], "curses"))
          {
            gui = NCURSES;
            i++;
          } else if (!strcmp(arg[i + 1], "gtk"))
          {
            gui = GTK;
            i++;
          } else
          {
            usage();
            return 0;
          }
          break;
        default:
          {
            usage();
            return 0;
          }
      }
    } else
    {
      usage();
      return 1;
    }
  }

#ifdef USE_GTK
  if (!THEME)
    THEME = strdup("Default");
#endif

  logg("main - After parsing argouments mode = %d\n", mode);

  // to init _malloc allocation functions
  logg("main - before _malloc_init\n");
  _malloc_init(getpid());

  // check if configuration file exist
  logg("main - check if configuration file exist\n");
  snprintf(buff, BUFFER_LEN, "%s/.paranoyrc",
           getenv("HOME") ? getenv("HOME") : "");

  logg("main - open configuration file...\n");
  if ((conf = fopen(buff, "r")) == NULL)
  {

    // file exists but we can't open it
    if (errno != ENOENT)
    {
      snprintf(buff, BUFFER_LEN, "Error while open configuration file: %s",
               strerror(errno));
      end(buff);
    }
    // this is the first time we run paranoy
    first_time();

  } else
  {
    // read configuration
    logg("main - before read_configuration\n");
    read_configuration(conf);
  }



  // load emails from disk
  logg("main - before parse_folder\n");
  parse_folder();
  logg("main - after parse_folder\n");
  // name of local machine


  init_queue(&infoq);

  // lunch thread that download email each n seconds
  pthread_create(&sleep_thread, NULL, (void *) timed, NULL);

  // lunch info thread
  pthread_mutex_init(&info_mutex, NULL);
  pthread_cond_init(&info_cond, NULL);
  pthread_create(&info_thread, NULL, (void *) infoz, NULL);

  gethostname(buff, BUFFER_LEN);
  localhost = buff ? strdup(buff) : strdup("localhost");

  init_queue(&infoq);

  connect_timeout = 4;

  if (mode == COMPOSE && smtpserver == NULL)
    end("No smtp server configured\n");


  if (gui == NCURSES)
#ifdef USE_CURSES
    return (curmain(narg, arg));
#else
  {
    fprintf(stderr, "The curses interface is disabled at compile time...\n");
    return -1;
  }
#endif
  else
  {
#ifdef USE_GTK
    if (getenv("DISPLAY"))
      return gtkmain(narg, arg);
    else
    {
      fprintf(stderr, "X does not run...\n");
#ifdef USE_CURSES
      fprintf(stderr, "Starting in ncurses interface...\n");
      sleep(2);
      return (curmain(narg, arg));
#endif
    }
#else
    fprintf(stderr, "The gtk interface is disabled at compile time...\n");
    return -1;
#endif
  }
}
